package com.easou.users.analysis.buildreport;

import java.io.*;
import java.util.*;

import android.content.*;

import com.easou.users.analysis.collect.*;
import com.easou.users.analysis.common.*;
import com.easou.users.analysis.entity.*;

/**
 * @author 赵自强（shenbaise1001@126.com）
 * @desc 负责压缩数据并存储到本地文件[记录日志]
 */
public class ReportBuilder {
	
	private final static String	LOG_TAG	= ReportBuilder.class.getName();
	String						filePath;
	Context						context;
	
	/**
	 * @param context
	 * @param logType -- 日志类型，见CommonConfig中定义的TYPE_XXXINFO的定义
	 */
	public ReportBuilder(Context context,int logType) {
		super();
		this.filePath = genLogFile(context,logType);
		this.context = context;
	}
	
	/**
	 * 将buffer写入到本地文件并记录到report数据表
	 * 
	 * @param buffer
	 * @param reprotType
	 * @return
	 */
	public boolean buildReport(byte[] buffer, int reprotType) {
		if (createFile() == true) {
			if ((writeData2File(buffer) == true) && new File(filePath).exists()) {
				if (reprotType == CommonConfig.TYPE_CUSTOMLOG) {
					DatabaseAdapater.getDatabaseAdapater(context).insert(new Report(CommonUtil.getCustomLogHost(context), filePath, reprotType));
				} else {
				    DatabaseAdapater.getDatabaseAdapater(context).insert(new Report(CommonUtil.getUpLoadUserInfoPath(context), filePath, reprotType));
				}
				return true;
			} else
				return false;
		}
		return false;
	}
	
	/**
	 * 将日志写入文件，返回文件句柄。
	 * 
	 * @param context
	 * @param buffer
	 * @param reportType
	 * @return
	 */
	public File buildReportFile(byte[] buffer,int reportType){
		String filePath = genLogFile(context, reportType);
		try {
			File file = new File(filePath);
			if ((writeData2File(buffer,file) == true) && new File(filePath).exists()) {
				return file;
			}else {
				return null;
			}
		} catch (Exception e) {
		}
		return null;
	}
	
	/**
	 * 将str写入到本地文件，并记录到report数据表
	 * 
	 * @param str
	 * @param reprotType
	 * @return
	 */
	public boolean buildReport(String str, int reprotType) {
		if (createFile() == true) {
			if ((writeData2File(str) == true) && new File(filePath).exists()) {
				DatabaseAdapater.getDatabaseAdapater(context).insert(new Report(CommonUtil.getUpLoadUserInfoPath(context), filePath, reprotType));
				return true;
			} else
				return false;
		}
		return false;
	}
	
	/**
	 * 创建文件
	 * 
	 * @return
	 */
	private boolean createFile() {
		File file = new File(filePath);
		if (!file.exists()) {
			try {
				file.createNewFile();
				return true;
			} catch (IOException e) {
				CommonUtil.printELog(LOG_TAG + "-createFile", "", e);
				return false;
			}
		}
		return true;
	}
	
	/**
	 * 写入本地文件
	 * 
	 * @param str
	 * @return
	 */
	private boolean writeData2File(String str) {
		FileWriter fileWriter = null;
		BufferedWriter bufferedWriter = null;
		try {
			fileWriter = new FileWriter(filePath, true);
			bufferedWriter = new BufferedWriter(fileWriter);
			bufferedWriter.write(str);
			bufferedWriter.flush();
			bufferedWriter.close();
			fileWriter.close();
		} catch (IOException e) {
			CommonUtil.printELog(LOG_TAG + "-writeData2File", "", e);
			return false;
		}finally{
			try {
				bufferedWriter.close();
				fileWriter.close();
			} catch (IOException e1) {
				CommonUtil.printELog(LOG_TAG + "-writeData2File", "", e1);
			}
		}
		return true;
	}
	
	/**
	 * 写入本地文件
	 * 
	 * @param buffer
	 * @return
	 */
	private boolean writeData2File(byte[] buffer) {
		File file = new File(filePath);
		FileOutputStream fileOutputStream = null;
		try {
			fileOutputStream = new FileOutputStream(file);
			fileOutputStream.write(buffer);
			fileOutputStream.flush();
		} catch (FileNotFoundException e) {
			CommonUtil.printELog(LOG_TAG + "-writeData2File", "", e);
		} catch (IOException e) {
			CommonUtil.printELog(LOG_TAG + "-writeData2File", "", e);
		} finally {
			if (fileOutputStream != null)
				try {
					fileOutputStream.close();
				} catch (IOException e) {
					CommonUtil.printELog(LOG_TAG + "-writeData2File", "", e);
				}
		}
		return true;
	}
	
	/**
	 * 写入文件
	 * 
	 * @param buffer
	 * @param file
	 * @return
	 */
	private boolean writeData2File(byte[] buffer,File file){
		FileOutputStream fileOutputStream = null;
		try {
			fileOutputStream = new FileOutputStream(file);
			fileOutputStream.write(buffer);
			fileOutputStream.flush();
		} catch (FileNotFoundException e) {
			CommonUtil.printELog(LOG_TAG + "-writeData2File", "", e);
		} catch (IOException e) {
			CommonUtil.printELog(LOG_TAG + "-writeData2File", "", e);
		} finally {
			if (fileOutputStream != null)
				try {
					fileOutputStream.close();
				} catch (IOException e) {
					CommonUtil.printELog(LOG_TAG + "-writeData2File", "", e);
				}
		}
		return true;
	}
	
	/**
	 * 生成文件名，文件名组成user_log_日志类型对应的数字_5个随机字符_时间戳
	 * 
	 * @param context
	 * @return
	 */
	private String genLogFile(Context context,int logType) {
		return new StringBuffer().append(context.getFilesDir()).append("/")
				.append("user_log_").append(logType).append("_")
				.append(UUID.randomUUID().toString().subSequence(0, 5)).append("_")
				.append(System.currentTimeMillis()).toString();
	}
	
	public boolean genRealTimeLogPath(String info){
		if (createFile() == true) {
			if ((writeData2File(GZip.compress_(context,info)) == true) && new File(this.filePath).exists()) {
				return true;
			}
		}
		return false;
	}
	
	public String getFilePath(){
		return this.filePath;
	}
	
}