/**
 * @工程 DataCollect
 * @文件 AbsLogCollect.java
 * @时间 2013-12-24 下午4:55:42
 * @作者  赵自强（shenbaise1001@126.com）
 * @描述 
 */
package com.easou.users.analysis.collect;

import java.util.List;

import android.content.Context;

import com.easou.users.analysis.common.CommonConfig;
import com.easou.users.analysis.common.CommonUtil;
import com.easou.users.analysis.common.Excutor;
import com.easou.users.analysis.common.OpenUDID;
import com.easou.users.analysis.common.Send;
import com.easou.users.analysis.entity.LogDataInterFace;

/**
 * @author 赵自强（shenbaise1001@126.com）
 * @desc
 */
public abstract class AbsLogCollect<T extends LogDataInterFace> {
//	private final static String	LOG_TAG	= AbsLogCollect.class.getName();
	private static Object lock = new Object();
	/**
	 * 日志类型
	 */
	protected int logType = -1;
	/**
	 * 构造方法
	 * @param logType	-- 日志类型 详见CommonConfig中
	 */
	public AbsLogCollect(int logType){
		this.logType = logType;
	}
	/**
	 * 收集日志信息，适当时候上传服务器。
	 * @param context
	 * @param c
	 * @param e
	 */
	public void collect(final Context context, final T logDatas,final boolean force){
		Excutor.submit(new Runnable() {
			@Override
			public void run() {
				synchronized (lock) {
					// 入库
					saveLogDataToDb(context.getApplicationContext(), logDatas);
					List<T> data = getLogDatas(context.getApplicationContext());
					if (data != null && !data.isEmpty()) {
						makeFileAndUploadLog(context, data, force);
					}
				}
			}
		});
	}
	/**
	 * 上传服务器。
	 * @param context
	 * @param logDatas
	 * @param force
	 */
	public void collect(final Context context, final boolean force){
		Excutor.submit(new Runnable() {
			@Override
			public void run() {
				synchronized (lock) {
					List<T> data = getLogDatas(context.getApplicationContext());
					if (data != null && !data.isEmpty()) {
						makeFileAndUploadLog(context, data, force);
					}
				}
			}
		});
	}
	
	/**
	 * 收集日志信息，适当时候上传服务器。上传时同步方法进行。
	 * @param context
	 * @param logDatas
	 */
	public void collectSyn(final Context context, final T logDatas){
		synchronized (lock) {
			// 入库
			saveLogDataToDb(context.getApplicationContext(), logDatas);
			List<T> data = getLogDatas(context.getApplicationContext());
			if (data != null && !data.isEmpty()) {
				makeFileAndUploadLog(context,data,true);
			}
		}
	}
	/**
	 * 1、生成本地文件，记录到report表。<br>
	 * 2、遍历report表中对应的LogType的日志记录。<br>
	 * 3、上传记录中的文件->删除记录。<br>
	 * 4、删除日志数据表中相应的日志信息。
	 * @param context
	 * @param data
	 * @param force --表示是否超时，如果超时算一次启动。
	 */
	private void makeFileAndUploadLog(final Context context,List<T> data,boolean force){
		synchronized (lock) {
			if(null==context)
				return;
			Context aContext = context.getApplicationContext();
			OpenUDID.syncContext(aContext);
			int n = CommonUtil.getCountLimited2Report(aContext, CommonConfig.CountLimited2Report);
			if(force || (null!=data && data.size() >= n) || CommonUtil.isNetworkAvailable(aContext) && CommonUtil.CurrentNoteworkTypeIsWIFI(aContext)){
				if(makeLogReport(context, data)){
					delateDataLogFromDb(aContext);
					int m = Send.getSend().getFileNum(aContext);
					if(m>=n)
						force = true;
					//网络可用且是wifi实时上传activity日志
					if(CommonUtil.isNetworkAvailable(aContext) && CommonUtil.CurrentNoteworkTypeIsWIFI(aContext)){
						//wifi条件下实时上传
						Send.getSend().sendReport(aContext,logType);
					}else if (CommonUtil.isNetworkAvailable(aContext) && data.size() >= n) {
						//网络可用，数据条数大于10条，强制上传
						Send.getSend().sendReport(aContext,logType);
					}else if (CommonUtil.isNetworkAvailable(aContext) && force) {
						//网络可用，人为强制上传
						Send.getSend().sendReport(aContext,logType);
					}
				}else {
					CommonUtil.printDLog(this.getClass().getName(), "生成本地文件时失败");
				}
			}
		}
	}
	/**
	 * 保存日志到数据表
	 * @param context
	 * @param name
	 * @param e
	 */
	protected abstract boolean saveLogDataToDb(final Context context, T logDate);
	/**
	 * 删除相应日志数据
	 * @param context
	 * @return
	 */
	protected abstract void delateDataLogFromDb(final Context context);
	/**
	 * 删除相应日志数据
	 * @param ids
	 */
	//protected abstract void delateDataLogFromDb(Context context,final List<String> ids);
	/**
	 * 获取现有全部日志信息
	 * @return
	 */
	protected abstract List<T> getLogDatas(Context context);
	/**
	 * 将日志写入到本地文件并记录在report表
	 * @param context
	 * @param elist
	 * @return
	 */
	protected abstract boolean makeLogReport(Context context, List<T> logDate);
}
