package com.easou.users.analysis.collect;

import java.util.List;

import org.json.JSONArray;
import org.json.JSONException;
import org.json.JSONObject;

import android.content.Context;

import com.easou.users.analysis.DataCollect;
import com.easou.users.analysis.buildreport.ReportBuilder;
import com.easou.users.analysis.common.CommonConfig;
import com.easou.users.analysis.common.CommonUtil;
import com.easou.users.analysis.common.GZip;
import com.easou.users.analysis.common.OpenUDID;
import com.easou.users.analysis.entity.ActivityEntity;
import com.easou.users.analysis.mobilephoneInfo.MobilePhoneInfo;
import com.easou.users.analysis.mobilephoneInfo.PhoneParameter;

public class ActivityCollect extends AbsLogCollect<ActivityEntity> {

	public ActivityCollect(int logType) {
		super(logType);
	}

	private final static String LOG_TAG = ActivityCollect.class.getName();

	/**
	 * add by andrew 统计activity时长
	 * 
	 * @param context
	 * @param a
	 */
	public void updateActivity(final Context context, ActivityEntity a) {
		DatabaseAdapater.getDatabaseAdapater(context).insert(a);
	}

	/**
	 * @Title: getActivitiesInfoJSONString
	 * @Description: Activity行为日志
	 * @param context
	 * @param bjsonArray
	 * @param mobilePhoneInfo
	 * @return
	 * @author vic
	 * @return JSONObject
	 * @throws
	 */
	private static JSONObject getActivitiesInfoJSONString(Context context,
			JSONArray ajsonArray) {
		JSONObject behaviorInfo = new JSONObject();
		MobilePhoneInfo mpinfo = new MobilePhoneInfo(context);
		try {
			behaviorInfo.put("appkey", CommonUtil.getAppKey(context));
			// behaviorInfo.put("cpid", CommonUtil.getChannelId(context));
			behaviorInfo.put("cpid", CommonUtil.getChannelIdRealTime(context));
			behaviorInfo.put(PhoneParameter.PHONE_SOFTVERSION, mpinfo
					.getSoftVersion() == null ? "" : mpinfo.getSoftVersion());
			behaviorInfo.put(CommonConfig.VERSION_CODE_FLAG,
					CommonConfig.VERSION_CODE);
			behaviorInfo.put("type", CommonConfig.TYPE_ACTIVITY_INFO);
			behaviorInfo.put("phone_udid", OpenUDID.getCorpUDID("com.easou"));
			behaviorInfo.put(
					"phone_udid2",
					OpenUDID.getCorpUDID2("com.easou") == null ? "" : OpenUDID
							.getCorpUDID2("com.easou"));
			behaviorInfo.put(PhoneParameter.PHONE_IMEI, mpinfo.getIMEI());
			behaviorInfo.put(PhoneParameter.PHONE_IMSI, mpinfo.getIMSI());
			behaviorInfo.put(PhoneParameter.PHONE_MAC, mpinfo.getMacAddress());
			behaviorInfo.put("currentnetworktype",
					CommonUtil.getNetworktype(context));
			behaviorInfo.put("gatewayip", CommonUtil.getGateWayIP(context));
			behaviorInfo.put(PhoneParameter.PHONE_APN,
					mpinfo.getAPN() == null ? "" : mpinfo.getAPN());
			behaviorInfo.put(PhoneParameter.PHONE_CITY,
					MobilePhoneInfo.getCity(context) == null ? ""
							: MobilePhoneInfo.getCity(context));	
			behaviorInfo.put("phone_esid", DataCollect.getSessionId(context));
			behaviorInfo.put("activities", ajsonArray);
			behaviorInfo
					.put("package_name", CommonUtil.getPackageName(context));
			behaviorInfo.put("last_cpid",
					CommonUtil.getLastChannelId(context) == null ? ""
							: CommonUtil.getLastChannelId(context));
			behaviorInfo.put(CommonConfig.OS, CommonConfig.ANDROID_OS);
			// 3.0 新增字段
			behaviorInfo.put("src_code", mpinfo.getSrcCode(context));
			behaviorInfo
					.put(PhoneParameter.PHONE_MODEL, mpinfo.getBuidlModel());
			behaviorInfo.put(PhoneParameter.PHONE_RESOLUTION,
					mpinfo.getResolution());
			behaviorInfo.put(PhoneParameter.PHONE_FIRMWARE_VERSION,
					mpinfo.getFirmwareVersion());
		} catch (JSONException e) {
			CommonUtil.printELog(LOG_TAG + "-getBehaviorInfoJSONString", "", e);
		}
		return behaviorInfo;
	}

	@Override
	public boolean saveLogDataToDb(Context context, ActivityEntity logDate) {
		if (logDate == null) {
			return false;
		}
		DatabaseAdapater.getDatabaseAdapater(context).insert(logDate);
		return true;
	}

	@Override
	protected List<ActivityEntity> getLogDatas(Context context) {
		return DatabaseAdapater.getDatabaseAdapater(context).getActivities();
	}

	@Override
	protected boolean makeLogReport(Context context,
			List<ActivityEntity> logDate) {
		if (null == logDate || logDate.isEmpty())
			return false;
		try {
			JSONArray ajsonArray = new JSONArray();
			for (ActivityEntity a : logDate) {
				JSONObject obj = new JSONObject();
				try {
					obj.put("time", a.getTime());
					obj.put("refer_activity", a.getRefer_activity());
					obj.put("current_activity", a.getCurrent_activity());
					obj.put("activity_use_time", a.getUse_time());
					try {
						obj.put("up_traffic", String.valueOf(a.upTraffic));
					} catch (Exception e) {
						CommonUtil.printELog(LOG_TAG + "-makeActivityReport",
								"", e);
					}
					try {
						obj.put("down_traffic", String.valueOf(a.downTraffic));
					} catch (Exception e) {
						CommonUtil.printELog(LOG_TAG + "-makeActivityReport",
								"", e);
					}
				} catch (JSONException e) {
					CommonUtil
							.printELog(LOG_TAG + "-makeActivityReport", "", e);
				}
				ajsonArray.put(obj);
			}
			JSONObject errorInfo = getActivitiesInfoJSONString(context,
					ajsonArray);
			ReportBuilder reportBuilder = new ReportBuilder(context,
					this.logType);
			String info = errorInfo.toString();
			CommonUtil.printDLog("easou_sdk",
					"activity --> " + errorInfo.toString());
			return reportBuilder.buildReport(GZip.compress_(context, info),
					CommonConfig.TYPE_ACTIVITYINFO);
		} catch (Exception e) {
		}
		return false;
	}

	@Override
	protected void delateDataLogFromDb(Context context) {
		DatabaseAdapater.getDatabaseAdapater(context).deleteActivities();
	}
}
