package com.easou.users.analysis.collect;

import java.util.List;

import org.json.JSONArray;
import org.json.JSONException;
import org.json.JSONObject;

import android.content.Context;

import com.easou.users.analysis.buildreport.ReportBuilder;
import com.easou.users.analysis.common.CommonConfig;
import com.easou.users.analysis.common.CommonUtil;
import com.easou.users.analysis.common.GZip;
import com.easou.users.analysis.common.OpenUDID;
import com.easou.users.analysis.entity.AppListEntity;

public class AppListCollect extends AbsLogCollect<AppListEntity> {
	private static final String LOG_TAG = AppListCollect.class.getName();
	
	public AppListCollect(int logType) {
		super(logType);
	}

	@Override
	protected boolean saveLogDataToDb(Context context, AppListEntity logDate) {
		if (logDate == null) {
			return false;
		}
		DatabaseAdapater.getDatabaseAdapater(context).insert(logDate);
		return true;
	}

	@Override
	protected void delateDataLogFromDb(Context context) {
		DatabaseAdapater.getDatabaseAdapater(context).deleteAppListLogs();
	}

	@Override
	protected List<AppListEntity> getLogDatas(Context context) {
		return DatabaseAdapater.getDatabaseAdapater(context).getAppListLogs();
	}

	@Override
	protected boolean makeLogReport(Context context, List<AppListEntity> logDate) {
		if(null==logDate || logDate.isEmpty())
			return false;
		for(AppListEntity ale : logDate){
			JSONObject jo = getAppListLogJSONString(context,ale);
			String info = jo.toString();
			CommonUtil.printDLog("easou_sdk", "applist_log = " + info);
			ReportBuilder reportBuilder = new ReportBuilder(context,this.logType);
			reportBuilder.buildReport(GZip.compress_(context,info),this.logType);
		}
		return true;
		
	}

	private JSONObject getAppListLogJSONString(Context context,
			AppListEntity ale) {
		
		JSONObject appListLog = new JSONObject();
		try {
			appListLog.put("type", CommonConfig.TYPE_APPLIST_INFO);
			appListLog.put("phone_udid", OpenUDID.getCorpUDID("com.easou"));
			appListLog.put("appkey", CommonUtil.getAppKey(context));
			appListLog.put("appUdid",ale.getAppUdid());
			appListLog.put("isUp",ale.getIsUp());
			appListLog.put("ctim",ale.getCtim());
			appListLog.put("paras",ale.getParas());
			
		} catch (JSONException e) {
			CommonUtil.printELog(LOG_TAG + "-getAppListLogJSONString", "", e);
		}
		return appListLog;
	}

}
