/**
 * @工程 DataCollect
 * @文件 BehaviorCollect.java
 * @时间 2013-12-25 下午3:29:57
 * @作者  赵自强（shenbaise1001@126.com）
 * @描述 
 */
package com.easou.users.analysis.collect;

import java.util.List;

import org.json.JSONArray;
import org.json.JSONException;
import org.json.JSONObject;

import android.content.Context;

import com.easou.users.analysis.DataCollect;
import com.easou.users.analysis.buildreport.ReportBuilder;
import com.easou.users.analysis.common.CommonConfig;
import com.easou.users.analysis.common.CommonUtil;
import com.easou.users.analysis.common.GZip;
import com.easou.users.analysis.common.OpenUDID;
import com.easou.users.analysis.entity.BehaviorEntitiy;
import com.easou.users.analysis.mobilephoneInfo.MobilePhoneInfo;

/**
 * @author 赵自强（shenbaise1001@126.com）
 * @desc
 */
public class BehaviorCollect extends AbsLogCollect<BehaviorEntitiy> {
	
	private final String LOG_TAG = BehaviorCollect.class.getName();

	public BehaviorCollect(int logType) {
		super(logType);
	}

	@Override
	protected void delateDataLogFromDb(Context context) {
		DatabaseAdapater.getDatabaseAdapater(context).deleteBehavior();
	}

	@Override
	protected List<BehaviorEntitiy> getLogDatas(Context context) {
		return DatabaseAdapater.getDatabaseAdapater(context).getBehaviors();
	}

	@Override
	protected boolean makeLogReport(Context context, List<BehaviorEntitiy> logDate) {
		if(null==logDate || logDate.isEmpty())
			return false;
		JSONArray bjsonArray = new JSONArray();
		for(BehaviorEntitiy entitiy:logDate){
			JSONObject opJ = new JSONObject();
			try {
				opJ.put("op", entitiy.getOp());
				opJ.put("info",
						new JSONObject(entitiy.getBehavior()));
				opJ.put("time", entitiy.getTime());
			} catch (JSONException e) {
				CommonUtil.printELog(LOG_TAG + "-submit", "", e);
				return false;
			}
		}
		MobilePhoneInfo mobilePhoneInfo = new MobilePhoneInfo(context);
		JSONObject behaviorInfo = getBehaviorInfoJSONString(context, bjsonArray, mobilePhoneInfo);
		CommonUtil.printDLog(LOG_TAG + "-submit",behaviorInfo.toString());
		ReportBuilder reportBuilder = new ReportBuilder(context,CommonConfig.TYPE_EVENTINFO);
		return reportBuilder.buildReport(GZip.compress_(context,behaviorInfo.toString()),CommonConfig.TYPE_USERINFO);
	}
	
	private JSONObject getBehaviorInfoJSONString(Context context,
			JSONArray bjsonArray, MobilePhoneInfo mobilePhoneInfo) {
		JSONObject behaviorInfo = new JSONObject();
		try {
			behaviorInfo.put("type", CommonConfig.TYPE_BEHAVIOR_INFO);
			behaviorInfo.put("phone_imei", MobilePhoneInfo.getIMEI(context));
			behaviorInfo.put("currentnetworktype",
					CommonUtil.getNetworktype(context));
			behaviorInfo.put("cpid", CommonUtil.getChannelId(context));
			behaviorInfo.put("phone_softversion",
					mobilePhoneInfo.getSoftVersion());
			behaviorInfo.put("phone_imsi", MobilePhoneInfo.getIMSI(context));
			behaviorInfo.put("phone_mac",
					MobilePhoneInfo.getMacAddress(context));
			behaviorInfo.put("appkey", CommonUtil.getAppKey(context));
			behaviorInfo.put("phone_city", MobilePhoneInfo.getCity(context));
			behaviorInfo.put("traffic", CommonUtil.getTraffic());
			behaviorInfo.put("phone_first_run",
					MobilePhoneInfo.isFirstStart(context));
			behaviorInfo.put("phone_udid", OpenUDID.getCorpUDID("com.easou"));
			behaviorInfo.put("behavior", bjsonArray);
			behaviorInfo.put("phone_esid", DataCollect.getSessionId(context));
			behaviorInfo.put("phone_apn", mobilePhoneInfo.getAPN() == null ? ""
					: mobilePhoneInfo.getAPN());
			behaviorInfo.put("package_name", mobilePhoneInfo.getSoftName() == null ? ""
					: mobilePhoneInfo.getSoftName());
			behaviorInfo.put("last_cpid", CommonUtil.getLastChannelId(context) == null ?  
					"" : CommonUtil.getLastChannelId(context));
			behaviorInfo.put(CommonConfig.OS, CommonConfig.ANDROID_OS);
		} catch (JSONException e) {
			CommonUtil.printELog(LOG_TAG + "-getBehaviorInfoJSONString", "", e);
		}
		return behaviorInfo;
	}
	
	@Override
	protected boolean saveLogDataToDb(Context context, BehaviorEntitiy logDate) {
		if(null==logDate)
			return false;
		DatabaseAdapater.getDatabaseAdapater(context).insert(logDate);
		return true;
	}
}
