package com.easou.users.analysis.collect;

import java.util.*;

import org.json.*;

import android.content.*;

import com.easou.users.analysis.buildreport.*;
import com.easou.users.analysis.common.*;
import com.easou.users.analysis.entity.*;
import com.easou.users.analysis.mobilephoneInfo.*;

public class CustomCollect extends AbsLogCollect<CustomEntitiy> {

	private static final String LOG_TAG = CustomCollect.class.getName();

	public CustomCollect(int logType) {
		super(logType);
	}

	@Override
	protected boolean saveLogDataToDb(Context context, CustomEntitiy logDate) {
		if (logDate == null) {
			return false;
		}
		DatabaseAdapater.getDatabaseAdapater(context).insert(logDate);
		return true;
	}

	@Override
	protected void delateDataLogFromDb(Context context) {
		DatabaseAdapater.getDatabaseAdapater(context).deleteCustomLogs();
	}

	@Override
	protected List<CustomEntitiy> getLogDatas(Context context) {
		return DatabaseAdapater.getDatabaseAdapater(context).getCustomLogs();
	}

	@Override
	protected boolean makeLogReport(Context context, List<CustomEntitiy> logDate) {
		if(null==logDate || logDate.isEmpty())
			return false;
		try {
			JSONArray bjsonArray = new JSONArray();
			for(CustomEntitiy entitiy:logDate){
				JSONObject opJ = new JSONObject();
				try {
					String info = entitiy.getCustom_log();
					if(info != null && info.length()>0 && !"".equals(info)){
						opJ.put("custom_log",new JSONObject(info));
					}
				} catch (JSONException e) {
					e.printStackTrace();
				}
				bjsonArray.put(opJ);
			}
			MobilePhoneInfo mobilePhoneInfo = new MobilePhoneInfo(context);
			
			JSONObject customLog = getCustomLogJSONString(context, bjsonArray, mobilePhoneInfo);
			String info = customLog.toString();
			CommonUtil.printDLog("easou_sdk", "custom_info = " + info);
			ReportBuilder reportBuilder = new ReportBuilder(context,this.logType);
			return reportBuilder.buildReport(GZip.compress_(context,info),this.logType);
		} catch (Exception e) {
			CommonUtil.printDLog(LOG_TAG, e.getMessage());
			return false;
		}
	}

	/**
	 * 生成event日志json
	 * 
	 * @param context
	 * @param bjsonArray
	 * @param mobilePhoneInfo
	 * @return
	 */
	private static JSONObject getCustomLogJSONString(Context context, JSONArray bjsonArray,
			MobilePhoneInfo mobilePhoneInfo) {
		JSONObject customLog = new JSONObject();
		try {
			customLog.put("custom_log", bjsonArray);
			customLog.put("type", CommonConfig.TYPE_CUSTOM_INFO);
			customLog.put("currentnetworktype", CommonUtil.getNetworktype(context));
			customLog.put("cpid", CommonUtil.getChannelId(context));
			customLog.put("phone_softversion", CommonUtil.getVersion(context));
			customLog.put("appkey", CommonUtil.getAppKey(context));
			customLog.put("phone_city", MobilePhoneInfo.getCity(context));
			customLog.put(CommonConfig.VERSION_CODE_FLAG, CommonConfig.VERSION_CODE);
			customLog.put("phone_udid", OpenUDID.getCorpUDID("com.easou"));
			customLog.put("phone_udid2", OpenUDID.getCorpUDID2("com.easou"));
			customLog.put(PhoneParameter.PHONE_IMEI, MobilePhoneInfo.getIMEI(context));
			customLog.put(PhoneParameter.PHONE_IMSI, MobilePhoneInfo.getIMSI(context));
			customLog.put(PhoneParameter.PHONE_MAC, MobilePhoneInfo.getMacAddress(context));
			customLog.put("gatewayip", CommonUtil.getGateWayIP(context));
			customLog.put("phone_apn", mobilePhoneInfo.getAPN() == null ? "" : mobilePhoneInfo.getAPN());
			customLog.put("package_name", mobilePhoneInfo.getSoftName());
			customLog.put("last_cpid",
					CommonUtil.getLastChannelId(context) == null ? "" : CommonUtil.getLastChannelId(context));
			customLog.put(CommonConfig.OS, CommonConfig.ANDROID_OS);
		} catch (JSONException e) {
			CommonUtil.printELog(LOG_TAG + "-getBehaviorInfoJSONString", "", e);
		}
		return customLog;
	}
}
