package com.easou.users.analysis.collect;

import java.util.ArrayList;
import java.util.List;

import android.content.Context;
import android.database.Cursor;
import android.database.SQLException;
import android.database.sqlite.SQLiteDatabase;
import android.database.sqlite.SQLiteOpenHelper;

import com.easou.users.analysis.SdkLock;
import com.easou.users.analysis.common.CommonConfig;
import com.easou.users.analysis.common.CommonUtil;
import com.easou.users.analysis.entity.ActivityEntity;
import com.easou.users.analysis.entity.AppListEntity;
import com.easou.users.analysis.entity.BehaviorEntitiy;
import com.easou.users.analysis.entity.CustomEntitiy;
import com.easou.users.analysis.entity.EventEntitiy;
import com.easou.users.analysis.entity.ExceptionEntity;
import com.easou.users.analysis.entity.Report;
import com.easou.users.analysis.entity.ShareLogEntitiy;

public class DatabaseAdapater extends DatabaseParameter {

	private static DatabaseHelper helper = null;
	private static DatabaseAdapater databaseAdapater = null;

	/**
	 * 获取数据库操作对象
	 * 
	 * @param context
	 * @return
	 */
	public static synchronized DatabaseAdapater getDatabaseAdapater(Context context) {
		if (null == databaseAdapater) {
			databaseAdapater = new DatabaseAdapater(context.getApplicationContext());
		} else if (null == DatabaseAdapater.helper) {
			databaseAdapater = new DatabaseAdapater(context.getApplicationContext());
		}
		return databaseAdapater;
	}

	/**
	 * 私有构造
	 * 
	 * @param context
	 */
	private DatabaseAdapater(Context context) {
		try {
			helper = new DatabaseHelper(context);
		} catch (Exception e) {
			CommonUtil.printELog(this.getClass().getName(), e.getMessage());
		}
	}

	/**
	 * 插入异常日志到数据库
	 * 
	 * @param dl
	 */
	public void insert(ExceptionEntity entitiy) {
		String sql = "insert into " + TABLE_EXCEPTIONS + " (className,type,message,stack,time) values(?,?,?,?,?)";
		synchronized (SdkLock.sqliteLock) {
			SQLiteDatabase db = null;
			try {
				Object[] parameterArray = new Object[] { entitiy.getClass(), entitiy.getType(), entitiy.getMessage(),
						entitiy.getStack() == null ? "" : entitiy.getStack(), entitiy.getTime() };
				db = helper.getWritableDatabase();
				db.execSQL(sql, parameterArray);
			} catch (SQLException e) {
				CommonUtil.printELog(this.getClass().getName(), e.getMessage());
			} finally {
				if (db != null) {
					db.close();
				}
			}
		}
	}

	/**
	 * 删除ID对应的report记录
	 * 
	 * @param id
	 */
	public void deleteReport(long id) {
		String sql = "delete from " + TABLE_REPORTS + " where id = ?";
		executeSql(sql, new Object[] { id });
	}

	public void deleteReport() {
		String sql = "delete from " + TABLE_REPORTS;
		doSql(sql);
	}

	/**
	 * 删除指定日期的异常信息
	 * 
	 * @param date
	 */
	public void deleteException(String date) {
		String sql = "delete from " + TABLE_EXCEPTIONS /*
														 * + " where date <> \"" + date + "\""
														 */;
		doSql(sql);
	}

	/**
	 * 删除全部异常
	 */
	public void deleteExceptions() {
		String sql = "delete from " + TABLE_EXCEPTIONS;
		doSql(sql);
	}

	/**
	 * 删除异常日志
	 * 
	 * @param ids
	 */
	public void deleteExceptions(List<String> ids) {
		String sql = "delete from " + TABLE_EXCEPTIONS + "where id in (?)";
		doSql(sql);
	}

	/**
	 * 清空相应日志的数据表
	 * 
	 * @param logType
	 */
	public void deleteLogDate(int logType) {
		String sql = "";
		switch (logType) {
		case CommonConfig.TYPE_ACTIVITYINFO:
			sql = "delete from " + TABLE_ACTIVITIES;
			doSql(sql);
			break;
		case CommonConfig.TYPE_EVENTINFO:
			sql = "delete from " + TABLE_EVENTINFO;
			doSql(sql);
			break;
		case CommonConfig.TYPE_EXCEPTIONINFO:
			sql = "delete from " + TABLE_EXCEPTIONS;
			doSql(sql);
			break;
		case CommonConfig.TYPE_BEHAVIORINFO:
			sql = "delete from " + TABLE_BEHAVIORS;
			doSql(sql);
			break;
		default:
			break;
		}
	}

	/**
	 * 获取指定日期<b>(date)</b>的异常
	 * 
	 * @param date
	 * @return
	 */
	@Deprecated
	public List<ExceptionEntity> getExceptionsByDate(String date) {
		List<ExceptionEntity> list = new ArrayList<ExceptionEntity>();
		String sql = "select * from " + TABLE_EXCEPTIONS /*
														 * + " where date <> \"" + date + "\""
														 */;
		synchronized (SdkLock.sqliteLock) {
			SQLiteDatabase db = null;
			Cursor c = null;
			try {
				db = helper.getReadableDatabase();
				c = db.rawQuery(sql, null);
				if (c == null) {
					return null;
				}
				if (c.getCount() == 0) {
					c.close();
					return null;
				}
				c.moveToFirst();
				do {
					int index = 0;
					ExceptionEntity dl = new ExceptionEntity();
					dl.setEid(c.getLong(index++));
					dl.setClassName(c.getString(index++));
					dl.setType(c.getString(index++));
					dl.setMessage(c.getString(index++));
					dl.setStack(c.getString(index++));
					dl.setTime(c.getString(index++));

					list.add(dl);
				} while (c.moveToNext());

			} catch (Exception e) {
			} finally {
				if (null != c && !c.isClosed())
					c.close();
				if (null != db)
					db.close();
			}
		}
		return list;
	}

	/**
	 * 获取全部异常信息
	 * 
	 * @return
	 */
	public List<ExceptionEntity> getExceptions() {
		List<ExceptionEntity> list = new ArrayList<ExceptionEntity>();

		String sql = "select * from " + TABLE_EXCEPTIONS /*
														 * + " where date <> \"" + date + "\""
														 */;

		synchronized (SdkLock.sqliteLock) {
			SQLiteDatabase db = null;
			Cursor c = null;
			try {
				db = helper.getReadableDatabase();
				c = db.rawQuery(sql, null);
				if (c == null) {
					return null;
				}
				if (c.getCount() == 0) {
					c.close();
					return null;
				}

				c.moveToFirst();
				do {
					int index = 0;
					ExceptionEntity dl = new ExceptionEntity();
					dl.setEid(c.getLong(index++));
					dl.setClassName(c.getString(index++));
					dl.setType(c.getString(index++));
					dl.setMessage(c.getString(index++));
					dl.setStack(c.getString(index++));
					dl.setTime(c.getString(index++));

					list.add(dl);
				} while (c.moveToNext());
			} catch (Exception e) {
			} finally {
				if (null != c && !c.isClosed())
					c.close();
				if (null != db)
					db.close();
			}
		}
		return list;
	}

	/**
	 * 执行sql，insert、update、delete等
	 * 
	 * @param sql
	 */
	public void doSql(String sql) {

		synchronized (SdkLock.sqliteLock) {
			SQLiteDatabase db = null;
			try {
				db = helper.getWritableDatabase();
				db.execSQL(sql);
			} catch (Exception e) {
				CommonUtil.printELog(this.getClass().getName(), e.getMessage());
			} finally {
				if (null != db)
					db.close();
			}
		}
	}

	/**
	 * 
	 * @param sql
	 * @param bindArgs
	 */
	public void executeSql(String sql, Object[] bindArgs) {
		synchronized (SdkLock.sqliteLock) {
			SQLiteDatabase db = null;
			try {
				db = helper.getWritableDatabase();
				db.execSQL(sql, bindArgs);
			} catch (SQLException e) {
				CommonUtil.printELog(this.getClass().getName(), e.getMessage());
			} finally {
				if (db != null) {
					db.close();
				}
			}
		}
	}

	public void insert(Report report) {
		String sql = "INSERT INTO " + TABLE_REPORTS + " (url,path,type) VALUES(?,?,?)";
		synchronized (SdkLock.sqliteLock) {
			SQLiteDatabase db = null;
			try {
				Object[] parameterArray = new Object[] { report.getUrl(), report.getPath(), report.getType() };
				db = helper.getWritableDatabase();
				db.execSQL(sql, parameterArray);
			} catch (SQLException e) {
				CommonUtil.printELog(this.getClass().getName(), e.getMessage());
			} finally {
				if (db != null) {
					db.close();
				}
			}
		}
	}

	/**
	 * 获取report记录
	 * 
	 * @return
	 */
	public List<Report> getReportList() {
		List<Report> list = new ArrayList<Report>();
		String sql = "select * from " + TABLE_REPORTS;
		synchronized (SdkLock.sqliteLock) {
			SQLiteDatabase db = null;
			Cursor cursor = null;
			try {
				db = helper.getWritableDatabase();
				cursor = db.rawQuery(sql, null);
				if (cursor == null) {
					return null;
				}
				if (cursor.getCount() == 0) {
					cursor.close();
					return null;
				}
				cursor.moveToFirst();
				do {
					int index = 0;
					Report dl = new Report();
					dl.setId(cursor.getLong(index++));
					dl.setUrl(cursor.getString(index++));
					dl.setPath(cursor.getString(index++));
					dl.setType(cursor.getInt(index++));
					list.add(dl);
				} while (cursor.moveToNext());
			} catch (Exception e) {
			} finally {
				if (null != cursor && !cursor.isClosed())
					cursor.close();
				if (db != null) {
					db.close();
				}
			}
		}
		return list;
	}

	/**
	 * 获取相应类型的日志记录
	 * 
	 * @param type
	 * @return
	 */
	public List<Report> getReportList(String type) {
		List<Report> list = new ArrayList<Report>();
		String sql = "select * from " + TABLE_REPORTS + " where type = ?";
		synchronized (SdkLock.sqliteLock) {
			SQLiteDatabase db = null;
			Cursor cursor = null;
			try {
				db = helper.getReadableDatabase();
				cursor = db.rawQuery(sql, new String[] { type });
				if (cursor == null) {
					return null;
				}
				if (cursor.getCount() == 0) {
					cursor.close();
					return null;
				}
				cursor.moveToFirst();
				do {
					int index = 0;
					Report dl = new Report();
					dl.setId(cursor.getLong(index++));
					dl.setUrl(cursor.getString(index++));
					dl.setPath(cursor.getString(index++));
					dl.setType(cursor.getInt(index++));
					list.add(dl);
				} while (cursor.moveToNext());
			} catch (Exception e) {
			} finally {
				if (null != cursor && !cursor.isClosed())
					cursor.close();
				if (null != db) {
					db.close();
				}
			}
		}
		return list;
	}

	/**
	 * 记录activity
	 * 
	 * @param a
	 */
	public void insert(ActivityEntity entitiy) {
		String sql = "insert into " + TABLE_ACTIVITIES
				+ " (time,current_activity,refer_activity,activity_use_time) values(?,?,?,?)";
		synchronized (SdkLock.sqliteLock) {
			SQLiteDatabase db = null;
			try {
				Object[] parameterArray = new Object[] { entitiy.getTime(), entitiy.getCurrent_activity(),
						entitiy.getRefer_activity(), entitiy.getUse_time() };
				db = helper.getWritableDatabase();
				db.execSQL(sql, parameterArray);
			} catch (SQLException e) {
				CommonUtil.printELog(this.getClass().getName(), e.getMessage());
			} finally {
				if (db != null) {
					db.close();
				}
			}
		}
	}

	/**
	 * 更新使用时间
	 * 
	 * @param a
	 */
	public void updateActivity(ActivityEntity a) {
		String sql = "update " + TABLE_ACTIVITIES + " set activity_use_time= " + a.getUse_time() + " , up = "
				+ a.upTraffic + " , down = " + a.downTraffic + " where id= " + a.getId();
		doSql(sql);
	}

	/**
	 * 插入Envent日志
	 * 
	 * @param entitiy
	 */
	public void insert(EventEntitiy entitiy) {
		String sql = "INSERT INTO " + DatabaseParameter.TABLE_EVENTINFO
				+ " (event_id,time,status,event_info) VALUES(?,?,?,?)";
		synchronized (SdkLock.sqliteLock) {
			SQLiteDatabase db = null;
			try {
				Object[] parameterArray = new Object[] { entitiy.getEvent_id(), entitiy.getTime(), entitiy.getStatus(),
						entitiy.getEvent_info() };
				db = helper.getWritableDatabase();
				db.execSQL(sql, parameterArray);
			} catch (SQLException e) {
				CommonUtil.printELog(this.getClass().getName(), e.getMessage());
			} finally {
				if (null != db) {
					db.close();
				}
			}
		}
	}

	/**
	 * 获取全部Envent日志
	 * 
	 * @return
	 */
	public List<EventEntitiy> getEnvents() {
		List<EventEntitiy> list = new ArrayList<EventEntitiy>();
		EventEntitiy dl = null;
		String sql = "select * from " + TABLE_EVENTINFO;
		synchronized (SdkLock.sqliteLock) {
			SQLiteDatabase db = null;
			Cursor cursor = null;
			try {
				db = helper.getReadableDatabase();
				cursor = db.rawQuery(sql, null);
				if (cursor == null) {
					return null;
				}
				if (cursor.getCount() == 0) {
					cursor.close();
					return null;
				}
				cursor.moveToFirst();
				do {
					int index = 0;
					dl = new EventEntitiy();
					dl.set_id(cursor.getLong(index++));
					dl.setEvent_id(cursor.getString(index++));
					dl.setTime(cursor.getString(index++));
					dl.setStatus(cursor.getString(index++));
					dl.setEvent_info(cursor.getString(index++));
					list.add(dl);
				} while (cursor.moveToNext());
			} catch (Exception e) {
			} finally {
				if (null != cursor && !cursor.isClosed())
					cursor.close();
				if (null != db) {
					db.close();
				}
			}
		}
		return list;
	}

	/**
	 * 删除表中event日志
	 */
	public void deleteEvents() {
		String sql = "delete from " + TABLE_EVENTINFO;
		doSql(sql);
	}

	/**
	 * 获取全部activities
	 * 
	 * @return
	 */
	public List<ActivityEntity> getActivities() {
		List<ActivityEntity> list = new ArrayList<ActivityEntity>();
		String sql = "select * from " + TABLE_ACTIVITIES;// +
															// " where activity_use_time !=null and activity_use_time !=''";

		synchronized (SdkLock.sqliteLock) {
			SQLiteDatabase db = null;
			Cursor c = null;
			try {
				db = helper.getReadableDatabase();
				c = db.rawQuery(sql, null);
				if (c == null) {
					CommonUtil.printELog("easou_sdk", "getActivities return null Cursor==null");
					return null;
				}
				if (c.getCount() == 0) {
					c.close();
					CommonUtil.printDLog("easou_sdk", "getActivities return null c.getCount()==0");
					return null;
				}

				c.moveToFirst();
				do {
					int index = 0;
					ActivityEntity dl = new ActivityEntity();
					dl.setId(c.getLong(index++));
					dl.setTime(c.getString(index++));
					dl.setCurrent_activity(c.getString(index++));
					dl.setRefer_activity(c.getString(index++));
					dl.setUse_time(c.getString(index++));
					try {
						String up = c.getString(index++);
						if (null == up || "".equals(up)) {
							up = "0";
						}
						dl.upTraffic = Long.valueOf(up);
					} catch (NumberFormatException e) {
						CommonUtil.printELog(this.getClass().getName(), e.getMessage());
					}
					try {
						String down = c.getString(index++);
						if (null == down || "".equals(down)) {
							down = "0";
						}
						dl.downTraffic = Long.valueOf(down);
					} catch (NumberFormatException e) {
						CommonUtil.printELog(this.getClass().getName(), e.getMessage());
					}
					list.add(dl);
				} while (c.moveToNext());
			} catch (Exception e) {
				CommonUtil.printELog(this.getClass().getName(), e.getMessage());
			} finally {
				if (null != c && !c.isClosed())
					c.close();
				if (null != db)
					db.close();
			}
		}
		return list;
	}

	/**
	 * 删除全部activities
	 */
	public void deleteActivities() {
		String sql = "delete from " + TABLE_ACTIVITIES;
		doSql(sql);
	}

	/**
	 * 记录行为日志
	 * 
	 * @param behavior
	 */
	public void insert(BehaviorEntitiy behavior) {
		String command = "INSERT INTO " + DatabaseParameter.TABLE_BEHAVIORS + " (op,behavior,time) VALUES(?,?,?)";
		synchronized (SdkLock.sqliteLock) {
			SQLiteDatabase db = null;
			try {
				db = helper.getWritableDatabase();
				Object[] parameterArray = new Object[] { (behavior.isFirsRun() ? 1 : 0) + "-" + behavior.getOp(),
						behavior.getBehavior(), String.valueOf(System.currentTimeMillis()) };
				db.execSQL(command, parameterArray);
			} catch (SQLException e) {
				CommonUtil.printELog(this.getClass().getName(), e.getMessage());
			} finally {
				if (null != db)
					db.close();
			}
		}
	}

	public List<BehaviorEntitiy> getBehaviors() {
		List<BehaviorEntitiy> list = null;
		String sql = "select * from " + TABLE_BEHAVIORS;
		synchronized (SdkLock.sqliteLock) {
			SQLiteDatabase db = null;
			Cursor c = null;
			try {
				db = helper.getReadableDatabase();
				c = db.rawQuery(sql, null);
				if (c == null) {
					return null;
				}
				if (c.getCount() == 0) {
					c.close();
					return null;
				}

				c.moveToFirst();
				list = new ArrayList<BehaviorEntitiy>();
				do {
					BehaviorEntitiy dl = new BehaviorEntitiy();
					dl.setId(c.getLong(0));
					dl.setOp(c.getString(c.getColumnIndex("op")));
					dl.setTime(c.getString(c.getColumnIndex("time")));
					dl.setBehavior(c.getString(c.getColumnIndex("behavior")));
					list.add(dl);
				} while (c.moveToNext());

			} catch (Exception e) {
				CommonUtil.printELog(this.getClass().getName(), e.getMessage());
			} finally {
				if (c != null && !c.isClosed())
					c.close();
				if (null != db)
					db.close();
			}
		}
		return list;
	}

	/**
	 * 删除全部行为日志
	 */
	public void deleteBehavior() {
		String sql = "delete from " + TABLE_BEHAVIORS;
		doSql(sql);
	}

	// ///////////// Share Log /////////////////
	public void insert(ShareLogEntitiy log) {
		String sql = "INSERT INTO "
				+ TABLE_SHARE
				+ " (id,status,time,share_page,share_sns,share_title,share_content,share_type,share_url,share_src,extra) VALUES(?,?,?,?,?,?,?,?,?,?,?)";
		synchronized (SdkLock.sqliteLock) {
			SQLiteDatabase db = null;
			try {
				Object[] parameterArray = new Object[] { log.getId(), log.getStatus(), log.getTime(),
						log.getShare_page(), log.getShare_sns(), log.getShare_title(), log.getShare_content(),
						log.getShare_type(), log.getShare_url(), log.getShare_src(), log.getExtra() };
				db = helper.getWritableDatabase();
				db.execSQL(sql, parameterArray);
			} catch (SQLException e) {
				CommonUtil.printELog(this.getClass().getName(), e.getMessage());
			} finally {
				if (db != null) {
					db.close();
				}
			}
		}
	}

	public ShareLogEntitiy getShareLogEntitiy(String id) {
		ShareLogEntitiy entitiy = null;
		String sql = "select * from " + TABLE_SHARE + " where id = '" + id + "'";
		synchronized (SdkLock.sqliteLock) {
			SQLiteDatabase db = null;
			Cursor c = null;
			try {
				db = helper.getReadableDatabase();
				c = db.rawQuery(sql, null);
				if (c == null) {
					return null;
				}
				if (c.getCount() == 0) {
					c.close();
					return null;
				}
				c.moveToFirst();
				do {
					entitiy = new ShareLogEntitiy();
					entitiy.setId(c.getString(c.getColumnIndex("id")));
					entitiy.setStatus(c.getString(c.getColumnIndex("status")));
					entitiy.setExtra(c.getString(c.getColumnIndex("extra")));
					entitiy.setShare_content(c.getString(c.getColumnIndex("share_content")));
					entitiy.setShare_page(c.getString(c.getColumnIndex("share_page")));
					entitiy.setShare_sns(c.getString(c.getColumnIndex("share_sns")));
					entitiy.setShare_src(c.getString(c.getColumnIndex("share_src")));
					entitiy.setShare_title(c.getString(c.getColumnIndex("share_title")));
					entitiy.setShare_type(c.getString(c.getColumnIndex("share_type")));
					entitiy.setShare_url(c.getString(c.getColumnIndex("share_url")));
					entitiy.setTime(c.getString(c.getColumnIndex("time")));
					break;
				} while (c.moveToNext());
			} catch (Exception e) {
				CommonUtil.printELog(this.getClass().getName(), e.getMessage());
			} finally {
				if (null != c && !c.isClosed())
					c.close();
				if (null != db)
					db.close();
			}
		}
		return entitiy;
	}

	public List<ShareLogEntitiy> getShareLogEntities() {
		List<ShareLogEntitiy> shares = new ArrayList<ShareLogEntitiy>();
		String sql = "select * from " + TABLE_SHARE;
		synchronized (SdkLock.sqliteLock) {
			SQLiteDatabase db = null;
			Cursor c = null;
			try {
				db = helper.getReadableDatabase();
				c = db.rawQuery(sql, null);
				if (c == null) {
					return null;
				}
				if (c.getCount() == 0) {
					c.close();
					return null;
				}
				c.moveToFirst();
				do {
					ShareLogEntitiy entitiy = new ShareLogEntitiy();
					entitiy.setId(c.getString(c.getColumnIndex("id")));
					entitiy.setStatus(c.getString(c.getColumnIndex("status")));
					entitiy.setExtra(c.getString(c.getColumnIndex("extra")));
					entitiy.setShare_content(c.getString(c.getColumnIndex("share_content")));
					entitiy.setShare_page(c.getString(c.getColumnIndex("share_page")));
					entitiy.setShare_sns(c.getString(c.getColumnIndex("share_sns")));
					entitiy.setShare_src(c.getString(c.getColumnIndex("share_src")));
					entitiy.setShare_title(c.getString(c.getColumnIndex("share_title")));
					entitiy.setShare_type(c.getString(c.getColumnIndex("share_type")));
					entitiy.setShare_url(c.getString(c.getColumnIndex("share_url")));
					entitiy.setTime(c.getString(c.getColumnIndex("time")));
					shares.add(entitiy);
				} while (c.moveToNext());
			} catch (Exception e) {
				CommonUtil.printELog(this.getClass().getName(), e.getMessage());
			} finally {
				if (null != c && !c.isClosed())
					c.close();
				if (null != db)
					db.close();
			}
		}
		return shares;
	}

	public void deleteShareLog(String id) {
		String sql = "delete from " + TABLE_SHARE + " where id = ?";
		executeSql(sql, new Object[] { id });
	}

	public void update(ShareLogEntitiy shareLog) {
		synchronized (SdkLock.sqliteLock) {
			deleteShareLog(shareLog.getId());
			insert(shareLog);
		}
	}
	
	/////////////////////////// custom log ////////////////////
	/**
	 * 插入 custom 日志
	 * @param entitiy
	 */
	public void insert(CustomEntitiy entitiy) {
		String sql = "INSERT INTO " + DatabaseParameter.TABLE_CUSTOM + " (custom_log) VALUES(?)";
		synchronized (SdkLock.sqliteLock) {
			SQLiteDatabase db = null;
			try {
				Object[] parameterArray = new Object[] {entitiy.getCustom_log() };
				db = helper.getWritableDatabase();
				db.execSQL(sql, parameterArray);
			} catch (SQLException e) {
				CommonUtil.printELog(this.getClass().getName(), e.getMessage());
			} finally {
				if (null != db) {
					db.close();
				}
			}
		}
	}
	
	/**
	 * 
	 * @return
	 */
	public void insert(AppListEntity entity) {
		String sql = "INSERT INTO " + DatabaseParameter.TABLE_APPLIST + " (appUdid,isUp,ctim,paras) VALUES(?,?,?,?)";
		synchronized (SdkLock.sqliteLock) {
			SQLiteDatabase db = null;
			try {
				Object[] parameterArray = new Object[] {entity.getAppUdid(),entity.getIsUp(),entity.getCtim(),entity.getParas()};
				db = helper.getWritableDatabase();
				db.execSQL(sql, parameterArray);
			} catch (SQLException e) {
				CommonUtil.printELog(this.getClass().getName(), e.getMessage());
			} finally {
				if (null != db) {
					db.close();
				}
			}
		}
		
	}
	
	public List<CustomEntitiy> getCustomLogs() {
		List<CustomEntitiy> list = null;
		String sql = "select * from " + TABLE_CUSTOM;
		synchronized (SdkLock.sqliteLock) {
			SQLiteDatabase db = null;
			Cursor c = null;
			try {
				db = helper.getReadableDatabase();
				c = db.rawQuery(sql, null);
				if (c == null) {
					return null;
				}
				if (c.getCount() == 0) {
					c.close();
					return null;
				}
				c.moveToFirst();
				list = new ArrayList<CustomEntitiy>();
				do {
					CustomEntitiy dl = new CustomEntitiy();
					dl.setId(c.getLong(0));
					dl.setCustom_log(c.getString(1));
					list.add(dl);
				} while (c.moveToNext());

			} catch (Exception e) {
				CommonUtil.printELog(this.getClass().getName(), e.getMessage());
			} finally {
				if (c != null && !c.isClosed())
					c.close();
				if (null != db)
					db.close();
			}
		}
		return list;
	}
	
	public void deleteCustomLog(String id) {
		String sql = "delete from " + TABLE_CUSTOM + " where id = ?";
		executeSql(sql, new Object[] { id });
	}
	
	public void deleteCustomLogs(){
		String sql = "delete from " + TABLE_CUSTOM;
		doSql(sql);
	}
	/**
	 * close
	 */
	public void close() {
		if (null != helper) {
			helper.close();
			helper = null;
		}
		if (databaseAdapater != null) {
			databaseAdapater.close();
			databaseAdapater = null;
		}
	}

	// ///////////////////////////////////////////////////////////////////// SQLiteOpenHelper

	public class DatabaseHelper extends SQLiteOpenHelper {
		public DatabaseHelper(Context context) {
			super(context, DATABASENAME, null, VERSION);
		}

		@Override
		public void onCreate(SQLiteDatabase db) {
			db.execSQL(CREATE_TABLE_REPORTS);
			db.execSQL(CREATE_TABLE_EXCEPTIONS);
			db.execSQL(CREATE_TABLE_ACTIVITIES);
			db.execSQL(CREATE_TABLE_BEHAVIORS);
			db.execSQL(CREATE_TABLE_EVENTINFO);
			db.execSQL(CREATE_TABLE_SHARE);
			db.execSQL(CREATE_TABLE_CUSTOM);
			db.execSQL(CREATE_TABLE_APPLIST);
		}

		@Override
		public void onUpgrade(SQLiteDatabase db, int oldVersion, int newVersion) {
			if (oldVersion != VERSION) {
				db.beginTransaction();
				db.execSQL(CREATE_TABLE_REPORTS);
				db.execSQL(CREATE_TABLE_EXCEPTIONS);
				db.execSQL(CREATE_TABLE_ACTIVITIES);
				db.execSQL(CREATE_TABLE_BEHAVIORS);
				db.execSQL(CREATE_TABLE_EVENTINFO);
				db.execSQL(CREATE_TABLE_SHARE);
				db.execSQL(CREATE_TABLE_CUSTOM);
				db.execSQL(CREATE_TABLE_APPLIST);
				db.setTransactionSuccessful();
				db.endTransaction();
				oldVersion = VERSION;
			}
		}

		@Override
		public synchronized void close() {
			super.close();
		}
	}

	public void deleteAppListLogs() {
		String sql = "delete from " + TABLE_APPLIST;
		doSql(sql);		
	}

	public List<AppListEntity> getAppListLogs() {
		List<AppListEntity> list = null;
		String sql = "select * from " + TABLE_APPLIST;
		synchronized (SdkLock.sqliteLock) {
			SQLiteDatabase db = null;
			Cursor c = null;
			try {
				db = helper.getReadableDatabase();
				c = db.rawQuery(sql, null);
				if (c == null) {
					return null;
				}
				if (c.getCount() == 0) {
					c.close();
					return null;
				}
				c.moveToFirst();
				list = new ArrayList<AppListEntity>();
				do {
					AppListEntity al = new AppListEntity();
					al.setAppUdid(c.getString(1));
					al.setIsUp(c.getString(2));
					al.setCtim(c.getLong(3));
					al.setParas(c.getString(4));
					list.add(al);
				} while (c.moveToNext());

			} catch (Exception e) {
				CommonUtil.printELog(this.getClass().getName(), e.getMessage());
			} finally {
				if (c != null && !c.isClosed())
					c.close();
				if (null != db)
					db.close();
			}
		}
		return list;
	}
}
