/**
 * @工程 DataCollect
 * @文件 EnventCollect.java
 * @时间 2013-12-24 下午7:13:14
 * @作者  赵自强（shenbaise1001@126.com）
 * @描述 
 */
package com.easou.users.analysis.collect;

import java.util.List;

import org.json.JSONArray;
import org.json.JSONException;
import org.json.JSONObject;

import android.content.Context;

import com.easou.users.analysis.buildreport.ReportBuilder;
import com.easou.users.analysis.common.CommonConfig;
import com.easou.users.analysis.common.CommonUtil;
import com.easou.users.analysis.common.GZip;
import com.easou.users.analysis.common.OpenUDID;
import com.easou.users.analysis.entity.EventEntitiy;
import com.easou.users.analysis.mobilephoneInfo.MobilePhoneInfo;
import com.easou.users.analysis.mobilephoneInfo.PhoneParameter;

/**
 * @author 赵自强（shenbaise1001@126.com）
 * @desc
 */
public class EnventCollect extends AbsLogCollect<EventEntitiy> {
	
	private static final String LOG_TAG = EnventCollect.class.getName();
	
	public EnventCollect(int logType) {
		super(logType);
	}

	@Override
	protected boolean saveLogDataToDb(Context context, EventEntitiy logDate) {
		if(logDate == null){
			return false;
		}
		DatabaseAdapater.getDatabaseAdapater(context).insert(logDate);
		return true;
	}

	@Override
	protected void delateDataLogFromDb(Context context) {
		DatabaseAdapater.getDatabaseAdapater(context).deleteEvents();
	}

	@Override
	protected List<EventEntitiy> getLogDatas(Context context) {
		return DatabaseAdapater.getDatabaseAdapater(context).getEnvents();
	}

	@Override
	protected boolean makeLogReport(Context context, List<EventEntitiy> logDate) {
		if(null==logDate || logDate.isEmpty())
			return false;
		try {
			JSONArray bjsonArray = new JSONArray();
			for(EventEntitiy entitiy:logDate){
				JSONObject opJ = new JSONObject();
				try {
					opJ.put("event_id", entitiy.getEvent_id());
					opJ.put("time", entitiy.getTime());
					opJ.put("status", entitiy.getStatus());
					String info = entitiy.getEvent_info();
					if(info != null && info.length()>0 && !"".equals(info)){					
						opJ.put("event_paralist",new JSONObject(info));
					}
				} catch (JSONException e) {
					e.printStackTrace();
				}
				bjsonArray.put(opJ);
			}
			MobilePhoneInfo mobilePhoneInfo = new MobilePhoneInfo(context);
			
			JSONObject eventInfo = getEventInfoJSONString(context, bjsonArray, mobilePhoneInfo);
			String info = eventInfo.toString();
			CommonUtil.printDLog("easou_sdk", "event_info = " + info);
			ReportBuilder reportBuilder = new ReportBuilder(context,this.logType);
			return reportBuilder.buildReport(GZip.compress_(context,info),this.logType);
		} catch (Exception e) {
			CommonUtil.printDLog(LOG_TAG, e.getMessage());
			return false;
		}
	}
	
	/**
	 * 生成event日志json 
	 * @param context
	 * @param bjsonArray
	 * @param mobilePhoneInfo
	 * @return
	 */
	// TODO 注释的部分是否需要？
	private static JSONObject getEventInfoJSONString(Context context,
			JSONArray bjsonArray, MobilePhoneInfo mobilePhoneInfo) {
		JSONObject behaviorInfo = new JSONObject();
		try {
			behaviorInfo.put("events", bjsonArray);
			behaviorInfo.put("type", CommonConfig.TYPE_EVENT_INFO);
			behaviorInfo.put("currentnetworktype",CommonUtil.getNetworktype(context));
			behaviorInfo.put("cpid", CommonUtil.getChannelId(context));
			behaviorInfo.put("phone_softversion",mobilePhoneInfo.getSoftVersion());
			behaviorInfo.put("appkey", CommonUtil.getAppKey(context));
		
			behaviorInfo.put("phone_city", MobilePhoneInfo.getCity(context));
			behaviorInfo.put(CommonConfig.VERSION_CODE_FLAG, CommonConfig.VERSION_CODE);
//			behaviorInfo.put("traffic", CommonUtil.getTraffic());
//			behaviorInfo.put("phone_first_run",
//					MobilePhoneInfo.isFirstStart(context));
			behaviorInfo.put("phone_udid", OpenUDID.getCorpUDID("com.easou"));
			behaviorInfo.put("phone_udid2", OpenUDID.getCorpUDID2("com.easou") == null ? "" : OpenUDID.getCorpUDID2("com.easou"));
			behaviorInfo.put(PhoneParameter.PHONE_IMEI, MobilePhoneInfo.getIMEI(context));
			behaviorInfo.put(PhoneParameter.PHONE_IMSI, MobilePhoneInfo.getIMSI(context));
			behaviorInfo.put(PhoneParameter.PHONE_MAC, MobilePhoneInfo.getMacAddress(context));
//			behaviorInfo.put("phone_esid", getSessionId(context));
			behaviorInfo.put("gatewayip", CommonUtil.getGateWayIP(context));
			behaviorInfo.put("phone_apn", mobilePhoneInfo.getAPN() == null ? ""
					: mobilePhoneInfo.getAPN());
			behaviorInfo.put("package_name", mobilePhoneInfo.getSoftName());
			behaviorInfo.put("last_cpid", CommonUtil.getLastChannelId(context) == null ?  
					"" : CommonUtil.getLastChannelId(context));
			behaviorInfo.put(CommonConfig.OS, CommonConfig.ANDROID_OS);
		} catch (JSONException e) {
			CommonUtil.printELog(LOG_TAG + "-getBehaviorInfoJSONString", "", e);
		}
		return behaviorInfo;
	}
}
