package com.easou.users.analysis.collect;

import java.util.List;

import org.json.JSONArray;
import org.json.JSONException;
import org.json.JSONObject;

import android.content.Context;
import android.os.Build;

import com.easou.users.analysis.buildreport.ReportBuilder;
import com.easou.users.analysis.common.CommonConfig;
import com.easou.users.analysis.common.CommonUtil;
import com.easou.users.analysis.common.GZip;
import com.easou.users.analysis.common.OpenUDID;
import com.easou.users.analysis.entity.ExceptionEntity;
import com.easou.users.analysis.mobilephoneInfo.MobilePhoneInfo;

/**
 * 
 * @author 赵自强（shenbaise1001@126.com）
 * @desc 异常日志管理
 * <b><i>注：</b></i> 此处异常上传时没有任何规则，直接进行上传。</br>
 * 现修规则为1、异常日志条数大于等于阈值时不论3g、wifi直接上传。</br>
 * 2、若在wifi环境下不检测阈值，有日志则进行上传。
 */
public class ExceptionCollect extends AbsLogCollect<ExceptionEntity>{
	
	public ExceptionCollect(int logType) {
		super(logType);
	}

	private final static String	LOG_TAG	= ExceptionCollect.class.getName();
	
	/**
	 * @Title: getErrorInfoJSONString
	 * @Description: 异常日志
	 * @param context
	 * @param ejsonArray
	 * @return
	 * @author vic
	 * @return JSONObject
	 * @throws
	 */
	@SuppressWarnings("static-access")
	private static JSONObject getErrorInfoJSONString(Context context, JSONArray ejsonArray) {
		JSONObject errorInfo = new JSONObject();
		try {
			Build bd = new Build();
			errorInfo.put("type", CommonConfig.TYPE_EXCEPTION_INFO);
			errorInfo.put("phone_udid", OpenUDID.getCorpUDID("com.easou"));
			errorInfo.put("phone_udid2", OpenUDID.getCorpUDID2("com.easou") == null ? "" : OpenUDID.getCorpUDID2("com.easou"));
			errorInfo.put("appkey", CommonUtil.getAppKey(context));
			errorInfo.put("phone_softversion", CommonUtil.getVersion(context));
			errorInfo.put("os_version", CommonUtil.getOsVersion(context));
			errorInfo.put("deviceid", bd.MANUFACTURER + bd.PRODUCT);
			errorInfo.put("phone_mac", MobilePhoneInfo.getMacAddress(context));
			errorInfo.put("phone_imei", MobilePhoneInfo.getIMEI(context));
			errorInfo.put("phone_imsi", MobilePhoneInfo.getIMSI(context));
			errorInfo.put("exception", ejsonArray);
			errorInfo.put("package_name", CommonUtil.getPackageName(context));
			errorInfo.put("last_cpid", CommonUtil.getLastChannelId(context) == null ? "" : CommonUtil.getLastChannelId(context));
			errorInfo.put("cpid", CommonUtil.getChannelId(context) == null ? "" : CommonUtil.getChannelId(context));
			errorInfo.put(CommonConfig.OS, CommonConfig.ANDROID_OS);
		} catch (JSONException e) {
			CommonUtil.printELog(LOG_TAG + "-getErrorInfoJSONString", "", e);
		}
		return errorInfo;
	}

	@Override
	protected boolean saveLogDataToDb(Context context,ExceptionEntity logDate) {
		try {
			if (logDate == null)
				return false;
			DatabaseAdapater.getDatabaseAdapater(context).insert(logDate);
			return true;
		} catch (Exception e) {
			return false;
		}
	}

	@Override
	protected List<ExceptionEntity> getLogDatas(Context context) {
		return DatabaseAdapater.getDatabaseAdapater(context).getExceptions();
	}

	@Override
	protected boolean makeLogReport(Context context,List<ExceptionEntity> logDate) {
		if(null==logDate || logDate.isEmpty())
			return false;
		JSONArray ejsonArray = new JSONArray();
		for (ExceptionEntity ee : logDate) {
			JSONObject obj = new JSONObject();
			try {
				obj.put("time", ee.getTime());
				obj.put("type", ee.getType());
				obj.put("stack", ee.getStack());
				obj.put("className", ee.getClassName());
			} catch (JSONException e) {
				CommonUtil.printELog(LOG_TAG + "-makeExceptionReport", "", e);
			}
			ejsonArray.put(obj);
		}
		JSONObject errorInfo = getErrorInfoJSONString(context, ejsonArray);
		ReportBuilder reportBuilder = new ReportBuilder(context,this.logType);
		return reportBuilder.buildReport(GZip.compress_(context, errorInfo.toString()), this.logType);
	}

	@Override
	protected void delateDataLogFromDb(Context context) {
		DatabaseAdapater.getDatabaseAdapater(context).deleteExceptions();
	}

//	@Override
//	protected void delateDataLogFromDb(Context context,List<String> ids) {
//		DatabaseAdapater.getDatabaseAdapater(context).deleteExceptions(ids);
//	}
	
}
