package com.easou.users.analysis.common;

import java.io.BufferedReader;
import java.io.IOException;
import java.io.InputStreamReader;
import java.io.UnsupportedEncodingException;
import java.net.MalformedURLException;
import java.net.URL;
import java.text.SimpleDateFormat;
import java.util.Calendar;
import java.util.Date;
import java.util.Locale;
import org.apache.http.HttpEntity;
import org.apache.http.HttpResponse;
import org.apache.http.client.ClientProtocolException;
import org.apache.http.client.HttpClient;
import org.apache.http.client.methods.HttpGet;
import org.apache.http.impl.client.DefaultHttpClient;
import org.json.JSONArray;
import org.json.JSONException;
import org.json.JSONObject;

import android.annotation.SuppressLint;
import android.content.Context;
import android.content.SharedPreferences;
import android.content.pm.ApplicationInfo;
import android.content.pm.PackageInfo;
import android.content.pm.PackageManager;
import android.content.pm.PackageManager.NameNotFoundException;
import android.hardware.SensorManager;
import android.location.Criteria;
import android.location.Location;
import android.location.LocationManager;
import android.net.ConnectivityManager;
import android.net.NetworkInfo;
import android.net.wifi.WifiInfo;
import android.net.wifi.WifiManager;
import android.os.AsyncTask;
import android.os.Process;
import android.telephony.TelephonyManager;
import android.text.TextUtils;
import android.text.format.Formatter;
import android.util.Log;

import com.easou.users.analysis.SdkLock;

@SuppressWarnings("deprecation")
@SuppressLint("SimpleDateFormat")
public class CommonUtil {

	private final static String LOG_TAG = CommonUtil.class.getName();

	/**
	 * checkPermissions
	 * 
	 * @param context
	 * @param permission
	 * @return true or false
	 */
	public static boolean checkPermissions(Context context, String permission) {
		try {
			PackageManager localPackageManager = context.getPackageManager();
			return localPackageManager.checkPermission(permission,
					context.getPackageName()) == PackageManager.PERMISSION_GRANTED;
		} catch (Exception e) {
		}
		return false;
	}

	/**
	 * Determine the current networking is WIFI
	 * 
	 * @param context
	 * @return
	 */
	public static boolean CurrentNoteworkTypeIsWIFI(Context context) {
		try {
			ConnectivityManager connectionManager = (ConnectivityManager) context
					.getSystemService(Context.CONNECTIVITY_SERVICE);
			return connectionManager.getActiveNetworkInfo().getType() == ConnectivityManager.TYPE_WIFI;
		} catch (Exception e) {
		}
		return false;
	}

	/**
	 * Judge wifi is available
	 * 
	 * @param inContext
	 * @return
	 */
	public static boolean isWiFiActive(Context inContext) {
		if (checkPermissions(inContext, "android.permission.ACCESS_WIFI_STATE")) {
			Context context = inContext.getApplicationContext();
			ConnectivityManager connectivity = (ConnectivityManager) context
					.getSystemService(Context.CONNECTIVITY_SERVICE);
			if (connectivity != null) {
				NetworkInfo[] info = connectivity.getAllNetworkInfo();
				if (info != null) {
					for (int i = 0; i < info.length; i++) {
						if (info[i].getTypeName().equals("WIFI")
								&& info[i].isConnected()) {
							return true;
						}
					}
				}
			}
			return false;
		} else {
			printELog(LOG_TAG + "-isWiFiActive",
					"lost permission--->android.permission.ACCESS_WIFI_STATE");
			return false;
		}
	}

	/**
	 * Testing equipment networking and networking WIFI
	 * 
	 * @param context
	 * @return true or false
	 */
	public static boolean isNetworkAvailable(Context context) {
		if (checkPermissions(context, "android.permission.INTERNET")) {
			ConnectivityManager cManager = (ConnectivityManager) context
					.getSystemService(Context.CONNECTIVITY_SERVICE);
			NetworkInfo info = cManager.getActiveNetworkInfo();
			if (info != null && info.isAvailable()) {
				return true;
			} else {
				printELog(LOG_TAG + "-isNetworkAvailable",
						"error:Network error");
				return false;
			}
		} else {
			printELog(LOG_TAG + "-isNetworkAvailable",
					"lost  permission----> android.permission.INTERNET");
			return false;
		}
	}

	/**
	 * Get the current time format yyyy-MM-dd HH:mm:ss
	 * 
	 * @return
	 */
	public static String getTime() {
		Date date = new Date();
		SimpleDateFormat localSimpleDateFormat = new SimpleDateFormat(
				"yyyy-MM-dd HH:mm:ss", Locale.US);
		// SimpleDateFormat localSimpleDateFormat = new
		// SimpleDateFormat("yyyy-MM-dd HH:mm:ss");
		return localSimpleDateFormat.format(date);
	}

	/**
	 * get APPKEY
	 * 
	 * @param context
	 * @return appkey
	 */
	public static String getAppKey(Context context) {
		String easouAppkey = "";
		try {
			PackageManager localPackageManager = context.getPackageManager();
			ApplicationInfo localApplicationInfo = localPackageManager
					.getApplicationInfo(context.getPackageName(), 128);
			if (localApplicationInfo != null) {
				String str = localApplicationInfo.metaData
						.getString("EASOU_APPKEY");
				if (str != null) {
					easouAppkey = str.toString();
				} else {
					printELog(LOG_TAG + "-getAppKey",
							"Could not read EASOU_APPKEY meta-data from AndroidManifest.xml");
				}
			}
		} catch (Exception e) {
			printELog(
					LOG_TAG + "-getAppKey",
					"Could not read EASOU_APPKEY meta-data from AndroidManifest.xml",
					e);
		}
		return easouAppkey;
	}

	/**
	 * 从androidManifest文件中获取metaData数据
	 * 
	 * @param context
	 * @param key
	 * @param def
	 * @return
	 */
	public static String getStringFromAppMetaData(Context context, String key,
			String def) {
		try {
			PackageManager localPackageManager = context.getPackageManager();
			ApplicationInfo localApplicationInfo = localPackageManager
					.getApplicationInfo(context.getPackageName(), 128);
			if (localApplicationInfo != null) {
				Object str = localApplicationInfo.metaData.get(key);
				if (null != str && !"".equals(str.toString()))
					return str.toString();
			}
		} catch (Exception e) {
			printELog(LOG_TAG, "Could not read " + key
					+ "meta-data from AndroidManifest.xml", e);
		}
		return def;
	}

	/**
	 * 从androidManifest文件中获取metaData数据
	 * 
	 * @param context
	 * @param key
	 * @param def
	 * @return
	 */
	public static Integer getIntFromAppMetaData(Context context, String key,
			int def) {
		try {
			PackageManager localPackageManager = context.getPackageManager();
			ApplicationInfo localApplicationInfo = localPackageManager
					.getApplicationInfo(context.getPackageName(), 128);
			if (localApplicationInfo != null) {
				return localApplicationInfo.metaData.getInt(key);
			}
		} catch (Exception e) {
			printELog(LOG_TAG, "Could not read " + key
					+ "meta-data from AndroidManifest.xml", e);
		}
		return def;
	}

	/**
	 * get Channel ID
	 * 
	 * @param context
	 * @return Channel ID
	 */
	public static String getChannelId(Context context) {
		String easouChannelId = "nice";
		SharedPreferences ps = context.getSharedPreferences(
				CommonConfig.COMMON_PREFS, Context.MODE_PRIVATE);
		easouChannelId = ps.getString(CommonConfig.CHANNEL, "");
		CommonUtil
				.printDLog("easou_sdk", "easouChannelId 1= " + easouChannelId);
		if (TextUtils.isEmpty(easouChannelId)) {

			easouChannelId = getChannelIdRealTime(context);

			// BufferedReader reader = null;
			// StringBuilder sb = new StringBuilder();
			// try {
			// reader = new BufferedReader(new
			// InputStreamReader(context.getResources().getAssets().open("cpid")));
			// String tempString = null;
			// while ((tempString = reader.readLine()) != null) {
			// sb.append(tempString);
			// }
			// reader.close();
			// } catch (IOException e) {
			// e.printStackTrace();
			// } finally {
			// if (reader != null) {
			// try {
			// reader.close();
			// } catch (IOException e1) {
			// }
			// }
			// }
			// String xx = sb.toString();//cpid=Neice
			// CommonUtil.printDLog("easou_sdk", "cpid = " + xx);
			// easouChannelId = xx;
			// if(!TextUtils.isEmpty(xx)){
			// easouChannelId = xx.substring(xx.indexOf("=") + 1);
			// }
			CommonUtil.printDLog("easou_sdk", "easouChannelId2 = "
					+ easouChannelId);
			setChannelId(context, easouChannelId);
		}
		return easouChannelId;
	}

	public static String getChannelIdRealTime(Context context) {
		String cpid = "nice";
		try {
			ApplicationInfo appInfo = null;
			appInfo = context.getPackageManager().getApplicationInfo(
					context.getPackageName(), PackageManager.GET_META_DATA);
			String channelKey = appInfo.metaData.getString("EasouBI_CHANNEL");
			if (channelKey != null && channelKey.trim().length() > 0) {
				cpid = appInfo.metaData.getString(channelKey.trim());
			}

		} catch (NameNotFoundException e) {
			printELog(LOG_TAG + "-getCpid", "", e);
		}

		return cpid;
		// String easouChannelId = "";
		// BufferedReader reader = null;
		// StringBuilder sb = new StringBuilder();
		// try {
		// reader = new BufferedReader(new
		// InputStreamReader(context.getResources().getAssets().open("cpid")));
		// String tempString = null;
		// while ((tempString = reader.readLine()) != null) {
		// sb.append(tempString);
		// }
		// reader.close();
		// } catch (IOException e) {
		// e.printStackTrace();
		// } finally {
		// if (reader != null) {
		// try {
		// reader.close();
		// } catch (IOException e1) {
		// }
		// }
		// }
		// String xx = sb.toString();//cpid=Neice
		// CommonUtil.printDLog("easou_sdk", "cpid = " + xx);
		// easouChannelId = xx;
		// if(!TextUtils.isEmpty(xx)){
		// easouChannelId = xx.substring(xx.indexOf("=") + 1);
		// }
		// CommonUtil.printDLog("easou_sdk", "easouChannelId2 = " +
		// easouChannelId);
		// return easouChannelId;
	}

	public static String getPreChannelId(Context context) {
		String easouChannelId = "";
		SharedPreferences ps = context.getSharedPreferences(
				CommonConfig.COMMON_PREFS, Context.MODE_PRIVATE);
		easouChannelId = ps.getString(CommonConfig.PRE_CHANNEL, "");
		return easouChannelId;
	}

	/**
	 * 获取上一个渠道
	 * 
	 * @param context
	 * @return
	 */
	public static String getLastChannelId(Context context) {
		String easouChannelId = "";
		SharedPreferences ps = context.getSharedPreferences(
				CommonConfig.COMMON_PREFS, Context.MODE_PRIVATE);
		easouChannelId = ps.getString(CommonConfig.LAST_CHANNEL, "");
		return easouChannelId;
	}

	public static void setChannelId(Context context, String channelId) {
		SharedPreferences ps = context.getSharedPreferences(
				CommonConfig.COMMON_PREFS, Context.MODE_PRIVATE);
		synchronized (SdkLock.localSharedPreferencesLock) {
			ps.edit().putString(CommonConfig.CHANNEL, channelId).commit();
		}
	}

	public static void setPreChannelId(Context context, String channelId) {
		SharedPreferences ps = context.getSharedPreferences(
				CommonConfig.COMMON_PREFS, Context.MODE_PRIVATE);
		synchronized (SdkLock.localSharedPreferencesLock) {
			ps.edit().putString(CommonConfig.PRE_CHANNEL, channelId).commit();
		}
	}

	/**
	 * 设置上一个渠道id
	 * 
	 * @param context
	 * @param channelId
	 */
	public static void setLastChannelId(Context context, String channelId) {
		SharedPreferences ps = context.getSharedPreferences(
				CommonConfig.COMMON_PREFS, Context.MODE_PRIVATE);
		synchronized (SdkLock.localSharedPreferencesLock) {
			ps.edit().putString(CommonConfig.LAST_CHANNEL, channelId).commit();
		}
	}

	/**
	 * 设置多少条日志日志记录上传一次（activity、event、exception日志）
	 * 
	 * @param context
	 * @param num
	 */
	public static void SetCountLimited2Report(Context context, Integer num) {
		SharedPreferences ps = context.getSharedPreferences(
				CommonConfig.COMMON_PREFS, Context.MODE_PRIVATE);
		synchronized (SdkLock.localSharedPreferencesLock) {
			ps.edit().putInt(CommonConfig.CountLimited2ReportStr, num).commit();
		}
	}

	/**
	 * 多少条上传一次
	 * 
	 * @param context
	 * @param defaultNum
	 * @return
	 */
	public static int getCountLimited2Report(Context context, Integer defaultNum) {
		SharedPreferences ps = context.getSharedPreferences(
				CommonConfig.COMMON_PREFS, Context.MODE_PRIVATE);
		return ps.getInt(CommonConfig.CountLimited2ReportStr,
				CommonConfig.CountLimited2Report);
	}

	/**
	 * get currnet activity's name
	 * 
	 * @param context
	 * @return
	 */
	public static String getActivityName(Context context) {
		if (context == null)
			return null;
		return context.getClass().getName();
		// ActivityManager am = (ActivityManager)
		// context.getSystemService(Context.ACTIVITY_SERVICE);
		// if (checkPermissions(context, "android.permission.GET_TASKS")) {
		// ComponentName cn = am.getRunningTasks(1).get(0).topActivity;
		// return cn.getShortClassName();
		// } else {
		// printELog(LOG_TAG + "-getActivityName",
		// "lost permission---->android.permission.GET_TASKS");
		// return null;
		// }
	}

	/**
	 * get PackageName
	 * 
	 * @param context
	 * @return
	 */
	public static String getPackageName(Context context) {
		if (context == null)
			// return "com.easou.plus";
			return "no package_name";
		return context.getPackageName();
		// ActivityManager am = (ActivityManager)
		// context.getSystemService(Context.ACTIVITY_SERVICE);
		//
		// if (checkPermissions(context, "android.permission.GET_TASKS")) {
		// ComponentName cn = am.getRunningTasks(1).get(0).topActivity;
		// return cn.getPackageName();
		// } else {
		// printELog(LOG_TAG + "-getPackageName",
		// "lost permission---->android.permission.GET_TASKS");
		// return null;
		// }
	}

	/**
	 * get OS number
	 * 
	 * @param context
	 * @return
	 */
	public static String getOsVersion(Context context) {
		String osVersion = "";
		if (checkPhoneState(context)) {
			osVersion = android.os.Build.VERSION.RELEASE;
			printDLog(LOG_TAG + "-getOsVersion", "OsVerson" + osVersion);
			return osVersion;
		} else {
			printELog(LOG_TAG + "-getOsVersion", "OsVerson get failed");
			return null;
		}
	}

	/**
	 * get deviceid
	 * 
	 * @param context
	 *            add <uses-permission android:name="READ_PHONE_STATE" />
	 * @return
	 */
	public static String getDeviceID(Context context) {
		if (checkPermissions(context, "android.permission.READ_PHONE_STATE")) {
			String deviceId = "";
			if (checkPhoneState(context)) {
				TelephonyManager tm = (TelephonyManager) context
						.getSystemService(Context.TELEPHONY_SERVICE);
				deviceId = tm.getDeviceId();
			}
			if (deviceId != null) {
				printDLog(LOG_TAG + "-getDeviceID", "deviceId:" + deviceId);
				return deviceId;
			} else {
				printELog(LOG_TAG + "-getDeviceID", "deviceId is null");
				return null;
			}
		} else {
			printELog(LOG_TAG + "-getDeviceID",
					"lost permissioin----->android.permission.READ_PHONE_STATE");
			return "";
		}
	}

	/**
	 * check phone _state is readied ;
	 * 
	 * @param context
	 * @return
	 */
	public static boolean checkPhoneState(Context context) {
		try {
			PackageManager packageManager = context.getPackageManager();
			if (packageManager.checkPermission(
					"android.permission.READ_PHONE_STATE",
					context.getPackageName()) != 0) {
				return false;
			}
		} catch (Exception e) {
		}
		return true;
	}

	/**
	 * get sdk number
	 * 
	 * @param paramContext
	 * @return
	 */
	public static String getSdkVersion(Context paramContext) {
		String osVersion = "";
		if (!checkPhoneState(paramContext)) {
			osVersion = android.os.Build.VERSION.RELEASE;
			printDLog(LOG_TAG + "-getSdkVersion", "OsVerson" + osVersion);
			return osVersion;
		} else {
			printELog(LOG_TAG + "-getSdkVersion", "OsVerson get failed");
			return null;
		}
	}

	/**
	 * Get the version number of the current program
	 * 
	 * @param context
	 * @return
	 */
	public static String getCurVersion(Context context) {
		String curversion = "";
		try {
			// ---get the package info---
			PackageManager pm = context.getPackageManager();
			PackageInfo pi = pm.getPackageInfo(context.getPackageName(), 0);
			curversion = pi.versionName;
			if (curversion == null || curversion.length() <= 0) {
				return "";
			}
		} catch (Exception e) {
			CommonUtil.printELog("VersionInfo", "Exception", e);
		}
		return curversion;
	}

	/**
	 * To determine whether it contains a gyroscope
	 * 
	 * @return
	 */
	public static boolean isHaveGravity(Context context) {
		try {
			SensorManager manager = (SensorManager) context
					.getSystemService(Context.SENSOR_SERVICE);
			if (manager == null) {
				return false;
			}
		} catch (Exception e) {
		}
		return true;
	}

	/**
	 * Get the current networking
	 * 
	 * @param context
	 * @return WIFI or MOBILE
	 */
	public static String getNetworkType(Context context) {
		TelephonyManager manager = (TelephonyManager) context
				.getSystemService(Context.TELEPHONY_SERVICE);
		int type = manager.getNetworkType();
		String typeString = "UNKOWN";
		if (type == TelephonyManager.NETWORK_TYPE_CDMA) {
			typeString = "CDMA";
		}
		if (type == TelephonyManager.NETWORK_TYPE_EDGE) {
			typeString = "EDGE";
		}
		if (type == TelephonyManager.NETWORK_TYPE_EVDO_0) {
			typeString = "EVDO_0";
		}
		if (type == TelephonyManager.NETWORK_TYPE_EVDO_A) {
			typeString = "EVDO_A";
		}
		if (type == TelephonyManager.NETWORK_TYPE_GPRS) {
			typeString = "GPRS";
		}
		if (type == TelephonyManager.NETWORK_TYPE_HSDPA) {
			typeString = "HSDPA";
		}
		if (type == TelephonyManager.NETWORK_TYPE_HSPA) {
			typeString = "HSPA";
		}
		if (type == TelephonyManager.NETWORK_TYPE_HSUPA) {
			typeString = "HSUPA";
		}
		if (type == TelephonyManager.NETWORK_TYPE_UMTS) {
			typeString = "UMTS";
		}
		if (type == TelephonyManager.NETWORK_TYPE_UNKNOWN) {
			typeString = "UNKOWN";
		}

		return typeString;
	}

	/**
	 * Determine the current network type
	 * 
	 * @param context
	 * @return
	 */
	public static boolean isNetworkTypeWifi(Context context) {

		if (checkPermissions(context, "android.permission.INTERNET")) {
			ConnectivityManager cManager = (ConnectivityManager) context
					.getSystemService(Context.CONNECTIVITY_SERVICE);
			NetworkInfo info = cManager.getActiveNetworkInfo();

			if (info != null && info.isAvailable()
					&& info.getTypeName().equals("WIFI")) {
				return true;
			} else {
				if (CommonConfig.DebugMode) {
					printELog(LOG_TAG + "-isNetworkTypeWifi",
							"error:Network not wifi");
				}
				return false;
			}
		} else {
			printELog(LOG_TAG + "-isNetworkTypeWifi",
					"lost  permission----> android.permission.INTERNET");
			return false;
		}

	}

	/**
	 * Get the current application version number
	 * 
	 * @param context
	 * @return
	 */
	public static String getVersion(Context context) {
		String version = "";
		String versionName = "";
		// int versionCode = -1;
		try {
			PackageManager pm = context.getPackageManager();
			PackageInfo pi = pm.getPackageInfo(context.getPackageName(), 0);
			versionName = pi.versionName;
			// versionCode = pi.versionCode;
			if (versionName == null
					|| (versionName != null && versionName.length() <= 0)) {
				versionName = "";
			}
			// if (versionCode > 0) {
			// version = versionName + versionCode;
			// } else {
			// version = versionName;
			// }
			version = versionName;
		} catch (Exception e) {
			printELog(LOG_TAG + "-getVersion", "", e);
		}
		return version;
	}

	public static void savePreVersion(Context context, String version) {
		SharedPreferences ps = context.getSharedPreferences(
				CommonConfig.COMMON_PREFS, Context.MODE_PRIVATE);
		synchronized (SdkLock.localSharedPreferencesLock) {
			ps.edit().putString(CommonConfig.PRE_APP_VERSION, version).commit();
		}
	}

	public static String getPreVersion(Context context) {
		String pre_version = "";
		SharedPreferences ps = context.getSharedPreferences(
				CommonConfig.COMMON_PREFS, Context.MODE_PRIVATE);
		pre_version = ps.getString(CommonConfig.PRE_APP_VERSION, "");
		return pre_version;
	}

	public static void setMInfoFlag(Context context, boolean flag) {
		SharedPreferences sharedPreferences = context.getSharedPreferences(
				CommonConfig.COMMON_PREFS, Context.MODE_PRIVATE);
		synchronized (SdkLock.localSharedPreferencesLock) {
			sharedPreferences.edit()
					.putBoolean(CommonConfig.MOBILE_PHONE_INFO_FLAG, flag)
					.commit();
		}
	}

	public static boolean getMInfoFlag(Context context) {
		SharedPreferences sharedPreferences = context.getSharedPreferences(
				CommonConfig.COMMON_PREFS, Context.MODE_PRIVATE);
		return sharedPreferences.getBoolean(
				CommonConfig.MOBILE_PHONE_INFO_FLAG, false);
	}

	public static String getYesterday() {
		Calendar cal = Calendar.getInstance();
		cal.add(Calendar.DATE, -1);

		SimpleDateFormat sdf = new SimpleDateFormat("yyyy-MM-dd");
		return sdf.format(cal.getTime());
	}

	public static String getCurrentDate() {
		SimpleDateFormat sdf = new SimpleDateFormat("yyyy-MM-dd");
		return sdf.format(new Date());
	}

	public static String getNetworktype(Context context) {
		int currentNet = -1;
		try {
			ConnectivityManager cm = (ConnectivityManager) context
					.getSystemService(Context.CONNECTIVITY_SERVICE);
			NetworkInfo net = cm.getActiveNetworkInfo();
			if (net != null) {
				if (!net.isAvailable())
					return CommonConfig.NONETWORK_FLAG;
				currentNet = net.getType();
				if (currentNet == CommonConfig.MOBILE) {
					// NETWORK_TYPE_EVDO_A是电信3G
					// NETWORK_TYPE_EVDO_A是中国电信3G的getNetworkType
					// NETWORK_TYPE_CDMA电信2G是CDMA
					// 移动2G卡 + CMCC + 2//type = NETWORK_TYPE_EDGE
					// 联通的2G经过测试 China Unicom 1 NETWORK_TYPE_GPRS
					String result = null;
					if (isConnectionFast(net.getSubtype())) {// 3G网络
						result = CommonConfig.G3_FLAG;
					} else if (net.getSubtype() == TelephonyManager.NETWORK_TYPE_LTE) {
						result = CommonConfig.G4_FLAG;
					} else {
						result = CommonConfig.G2_FLAG;
					}
					String mobileType = net.getExtraInfo();
					if (mobileType.contains(CommonConfig.WAP_FLAG)) {
						return result + CommonConfig.WAP_FLAG;
					} else if (mobileType.contains(CommonConfig.NET_FLAG)) {
						return result + CommonConfig.NET_FLAG;
					} else {
						return result;
					}
				} else if (currentNet == CommonConfig.WIFI)
					return CommonConfig.WIFI_FLAG;
			}
		} catch (Exception e) {
			CommonUtil.printELog(LOG_TAG + "-getNetworktype", "", e);
		}
		return CommonConfig.NONETWORK_FLAG;
	}

	private static boolean isConnectionFast(int subType) {
		switch (subType) {
		case TelephonyManager.NETWORK_TYPE_1xRTT:
			return false; // ~ 50-100 kbps
		case TelephonyManager.NETWORK_TYPE_CDMA:
			return false; // ~ 14-64 kbps
		case TelephonyManager.NETWORK_TYPE_EDGE:
			return false; // ~ 50-100 kbps
		case TelephonyManager.NETWORK_TYPE_EVDO_0:
			return true; // ~ 400-1000 kbps
		case TelephonyManager.NETWORK_TYPE_EVDO_A:
			return true; // ~ 600-1400 kbps
		case TelephonyManager.NETWORK_TYPE_GPRS:
			return false; // ~ 100 kbps
		case TelephonyManager.NETWORK_TYPE_HSDPA:
			return true; // ~ 2-14 Mbps
		case TelephonyManager.NETWORK_TYPE_HSPA:
			return true; // ~ 700-1700 kbps
		case TelephonyManager.NETWORK_TYPE_HSUPA:
			return true; // ~ 1-23 Mbps
		case TelephonyManager.NETWORK_TYPE_UMTS:
			return true; // ~ 400-7000 kbps
			// NOT AVAILABLE YET IN API LEVEL 7
			// case TelephonyManager.NETWORK_TYPE_EHRPD:
			// return true; // ~ 1-2 Mbps
		case TelephonyManager.NETWORK_TYPE_EVDO_B:
			return true; // ~ 5 Mbps
		case TelephonyManager.NETWORK_TYPE_HSPAP:
			return true; // ~ 10-20 Mbps
		case TelephonyManager.NETWORK_TYPE_IDEN:
			return false; // ~25 kbps
			// case TelephonyManager.NETWORK_TYPE_LTE:
			// return true; // ~ 10+ Mbps
			// Unknown
		case TelephonyManager.NETWORK_TYPE_UNKNOWN:
			return false;
		default:
			return false;
		}
	}

	/**
	 * 得到位置信息
	 */
	public static String getLocation(Context context) {
		try {
			// 获取位置管理服务
			LocationManager locationManager;
			String serviceName = Context.LOCATION_SERVICE;
			locationManager = (LocationManager) context
					.getSystemService(serviceName);
			// 查找到服务信息
			Criteria criteria = new Criteria();
			criteria.setAccuracy(Criteria.ACCURACY_FINE); // 高精度
			criteria.setAltitudeRequired(false);
			criteria.setBearingRequired(false);
			criteria.setCostAllowed(true);
			criteria.setPowerRequirement(Criteria.POWER_LOW); // 低功耗
			String provider = LocationManager.NETWORK_PROVIDER; // 获取GPS信息
			Location location = locationManager.getLastKnownLocation(provider); // 通过GPS获取位置
			if (location != null) {
				return getLocation(location, context);
			}
		} catch (Exception e) {
			CommonUtil.printELog(LOG_TAG + "-getLocation", "", e);
		}
		return "";
	}

	/**
	 * 根据精度和纬度定位到城市
	 * 
	 * @param itude
	 * @return
	 * @throws Exception
	 */
	private static String getLocation(final Location itude,
			final Context context) {
		final StringBuffer resultStr = new StringBuffer();
		new AsyncTask<String, String, String>() {

			@Override
			protected String doInBackground(String... params) {
				if (itude == null) {
					return "";
				}
				String resultString = "";
				/** 这里采用get方法，直接将参数加到URL上 */
				String urlString = String.format(
						CommonConfig.GOOGLE_GPS_ADDRESS, itude.getLongitude(),
						itude.getLatitude());

				/** 新建HttpClient */
				HttpClient client = new DefaultHttpClient();
				/** 采用GET方法 */
				HttpGet get = new HttpGet(urlString);
				try {

					/** 发起GET请求并获得返回数据 */
					HttpResponse response = client.execute(get);
					HttpEntity entity = response.getEntity();
					BufferedReader buffReader = new BufferedReader(
							new InputStreamReader(entity.getContent(),
									getEncode(entity)));
					StringBuffer strBuff = new StringBuffer();
					String result = null;
					while ((result = buffReader.readLine()) != null) {
						strBuff.append(result);
					}
					resultString = strBuff.toString();

					/** 解析JSON数据，获得物理地址 */
					if (resultString != null && resultString.length() > 0) {
						JSONObject jsonobject = new JSONObject(resultString);
						JSONArray jsonArray = new JSONArray(new JSONObject(
								jsonobject.get("detail").toString()).getString(
								"results").toString());
						resultString = "";
						for (int i = 0; i < jsonArray.length(); i++) {
							if (jsonArray.getJSONObject(i).getString("dtype")
									.equals("AD")) {
								resultString = judgeCity(jsonArray
										.getJSONObject(i).getString("name"));
								CommonUtil.printELog("resultString",
										resultString);
							}
						}
					}
				} catch (UnsupportedEncodingException e) {
					CommonUtil.printELog(LOG_TAG + "-getLocation", "", e);
				} catch (JSONException e1) {
					CommonUtil.printELog(LOG_TAG + "-getLocation", "", e1);
				} catch (ClientProtocolException e2) {
					CommonUtil.printELog(LOG_TAG + "-getLocation", "", e2);
				} catch (IOException e3) {
					CommonUtil.printELog(LOG_TAG + "-getLocation", "", e3);
				} finally {
					get.abort();
					client = null;
				}
				saveCity(context, resultString);
				return resultString;
			}

			@Override
			protected void onPostExecute(String result) {
				super.onPostExecute(result);
				resultStr.append(result);
			}

		}.execute("");
		return resultStr.toString();
	}

	private static String judgeCity(String city) {
		String result = "";
		if (city.contains("<html>") || city.contains("error")) {
			result = "";
		} else {
			result = city;
		}
		return result;
	}

	/**
	 * 根据httpEntity得到编码
	 * 
	 * @param entity
	 * @return
	 */
	private static String getEncode(HttpEntity entity) {
		String encode = "GBK";
		try {
			if (!(entity.getContentType() == null || entity.getContentType()
					.getValue() == null)) {
				String type = entity.getContentType().getValue();
				encode = type.substring(type.indexOf(("charset="))).split("=")[1];
			}
		} catch (Exception e) {
			CommonUtil.printELog(LOG_TAG + "-getEncode", "", e);
		}
		return encode;
	}

	/**
	 * 保存用户地区
	 * 
	 * @return
	 */
	public static void saveCity(Context context, String city) {
		if (city == null || city.trim().equals("")) {
			return;
		}
		SharedPreferences ps = context.getSharedPreferences(
				CommonConfig.COMMON_PREFS, Context.MODE_PRIVATE);
		synchronized (SdkLock.localSharedPreferencesLock) {
			ps.edit().putString(CommonConfig.ADDRESSS, city).commit();
		}
	}

	/**
	 * 读取地区
	 */
	public static String getCity(Context context) {
		SharedPreferences ps = context.getSharedPreferences(
				CommonConfig.COMMON_PREFS, Context.MODE_PRIVATE);
		return ps.getString(CommonConfig.ADDRESSS, "");
	}

	public static SimpleDateFormat formatDate = new SimpleDateFormat("yyyyMMdd");
	static String dateString = "";
	static String ipString = "";
	public static String getGateWayIP(Context context) {
		if (dateString.equals("") && ipString.equals("")) {
			dateString = formatDate.format(new Date());		
			ipString = getExtranetIP();
			return ipString;
		}
		if (dateString.equals(formatDate.format(new Date()))) {
			return ipString;
		} else {
			dateString = formatDate.format(new Date());
			ipString = getExtranetIP();
			return ipString;
		}
	}

	/**
	 * 获取外网IP
	 * @param context
	 * @return
	 */
	public static String getExtranetIP() {
		try {
			URL taobaoip = new URL("http://www.taobao.com/help/getip.php");
			BufferedReader taobaoin;
			try {
				taobaoin = new BufferedReader(new InputStreamReader(taobaoip.openStream()));
				String ips[] = taobaoin.readLine().split("\"");
				if (ips.length == 3) {
					return ips[1];
				}
			} catch (Exception e) {
				return "";
			}
		} catch (MalformedURLException e1) {
			return "";
		}
		return "";
	}
	
	
	public static String getWIFIIP(Context context) {
		WifiManager wifiManager = (WifiManager) context
				.getSystemService(Context.WIFI_SERVICE);
		WifiInfo wifiInfo = wifiManager.getConnectionInfo();
		int ipAddress = wifiInfo.getIpAddress();
		return Formatter.formatIpAddress(ipAddress);
	}

	/**
	 * 得到本软件的所有流量
	 * 
	 * @return
	 */
	public static long getTraffic() {
		long traffic = 0;
		try {
			int myUid = Process.myUid();
			traffic = TrafficMonitoring.monitoringEachApplicationReceive(myUid)
					+ TrafficMonitoring.monitoringEachApplicationSend(myUid);
		} catch (Exception e) {
			CommonUtil.printELog(LOG_TAG + "-getTraffic", "", e);
		}
		return traffic < 0 ? 0 : traffic;
	}

	/**
	 * Set the output Dlog
	 * 
	 * @param tag
	 * @param log
	 */
	public static void printDLog(String tag, String msg) {
		if (CommonConfig.DebugMode == true) {
			Log.d("EASOU_SDK-" + tag, msg);
		}
	}

	/**
	 * Set the output Elog
	 * 
	 * @param tag
	 * @param log
	 */
	public static void printELog(String tag, String msg, Throwable tr) {
		if (CommonConfig.DebugMode == true) {
			if (null != msg && null != tr)
				Log.e("EASOU_SDK-" + tag, msg, tr);
		}
	}

	/**
	 * Set the output Elog
	 * 
	 * @param tag
	 * @param log
	 */
	public static void printELog(String tag, String msg) {
		if (CommonConfig.DebugMode == true) {
			if (null != msg)
				Log.e("EASOU_SDK-" + tag, msg);
		}
	}

	// 保存城市信息
	public static void setCityInfo(Context context, String address) {
		CommonUtil.printDLog("easou_sdk", "setCityInfo---> address = "
				+ address);
		SharedPreferences ps = context.getSharedPreferences(
				CommonConfig.COMMON_PREFS, Context.MODE_PRIVATE);
		synchronized (SdkLock.localSharedPreferencesLock) {
			ps.edit().putString(CommonConfig.ADDRESSS, address).commit();
		}
	}

	// 保存经纬度信息
	public static void setLatitudeAndlongitude(Context context,
			double longitude, double latitude) {
		CommonUtil.printDLog("easou_sdk",
				"setLatitudeAndlongitude---> longitude = " + longitude
						+ "latitude = " + latitude);
		SharedPreferences ps = context.getSharedPreferences(
				CommonConfig.COMMON_PREFS, Context.MODE_PRIVATE);
		synchronized (SdkLock.localSharedPreferencesLock) {
			ps.edit()
					.putString(CommonConfig.LONGITUDE,
							String.valueOf(longitude)).commit();
			ps.edit()
					.putString(CommonConfig.LATITUDE, String.valueOf(latitude))
					.commit();
		}
	}

	public static double getLongitude(Context context) {
		SharedPreferences ps = context.getSharedPreferences(
				CommonConfig.COMMON_PREFS, Context.MODE_PRIVATE);
		synchronized (SdkLock.localSharedPreferencesLock) {
			String lo = ps.getString(CommonConfig.LONGITUDE, "0");
			double lo_ = Double.parseDouble(lo);
			return lo_;
		}
	}

	public static double getLatitude(Context context) {
		SharedPreferences ps = context.getSharedPreferences(
				CommonConfig.COMMON_PREFS, Context.MODE_PRIVATE);
		synchronized (SdkLock.localSharedPreferencesLock) {
			String la = ps.getString(CommonConfig.LATITUDE, "0");
			double la_ = Double.parseDouble(la);
			return la_;
		}
	}

	// 获取地区信息
	public static String getCityInfo(Context context) {
		SharedPreferences ps = context.getSharedPreferences(
				CommonConfig.COMMON_PREFS, Context.MODE_PRIVATE);
		return ps.getString(CommonConfig.ADDRESSS, "");
	}

	// 获取应用安装路径
	public static String getAppPath(Context context) {
		ApplicationInfo pm = null;
		try {
			pm = context.getPackageManager().getApplicationInfo(
					context.getPackageName(), PackageManager.GET_META_DATA);
			if (CommonConfig.DebugMode) {
				if (null != pm)
					android.util.Log.e("easou_sdk", "pm.sourceDir = "
							+ pm.sourceDir);
			}
		} catch (NameNotFoundException e) {
			e.printStackTrace();
		}
		return pm.sourceDir;
	}

	/**
	 * 格式化地区
	 * 
	 * @param address
	 * @return
	 */
	public static String formatAdrress(String address) {
		if (address == null) {
			return "";
		}
		StringBuffer sb = new StringBuffer();
		if (address.contains("省")) {
			int index_province = address.lastIndexOf("省");
			String province = address.substring(0, index_province + 1);
			sb.append(province);
			if (address.contains("市")) {
				int index_city = address.lastIndexOf("市");
				String city = address.substring(index_province + 1,
						index_city + 1);
				sb.append(",").append(city);
			}
		}
		return sb.toString();
	}

	/**
	 * 获取 EASOU_APPID_UPLOADUSERINFOPATH 地址
	 * 
	 * @param context
	 * @return
	 */
	public static String getUpLoadUserInfoPath(Context context) {
		String appId = CommonUtil.getStringFromAppMetaData(context,
				"EASOU_APPID", "");
		return ConfigLoader
				.getProperties(CommonConfig.EASOU_APPID_UPLOADUSERINFOPATH
						+ appId);
	}

	/**
	 * 获取 EASOU_APPID_CUSTOMLOGHOST 地址
	 * 
	 * @param context
	 * @return
	 */
	public static String getCustomLogHost(Context context) {
		String appId = CommonUtil.getStringFromAppMetaData(context,
				"EASOU_APPID", "");
		return ConfigLoader
				.getProperties(CommonConfig.EASOU_APPID_CUSTOMLOGHOST + appId);
	}
}
