package com.easou.users.analysis.common;

import java.io.ByteArrayInputStream;
import java.io.File;
import java.io.IOException;
import java.util.List;

import org.apache.http.HttpEntity;
import org.apache.http.HttpResponse;
import org.apache.http.HttpStatus;
import org.apache.http.client.ClientProtocolException;
import org.apache.http.client.methods.HttpPost;
import org.apache.http.entity.FileEntity;
import org.apache.http.entity.InputStreamEntity;
import org.apache.http.impl.client.DefaultHttpClient;
import org.apache.http.params.BasicHttpParams;
import org.apache.http.params.HttpConnectionParams;
import org.apache.http.params.HttpParams;

import android.content.Context;

import com.easou.users.analysis.collect.DatabaseAdapater;
import com.easou.users.analysis.entity.Report;
/**
 * 日志上传类
 * @author 赵自强（shenbaise1001@126.com）
 * @desc
 */
public class Send {
	
	private final static String	LOG_TAG	= Send.class.getName();
	private final static int connectionTimeOut = 30000;
	private final static int socketTimeout = 30000;
	
	private static Send send = null;
	private static Object lock = new Object();
	DefaultHttpClient defaultHttpClient = null;
	
	private Send(){
		HttpParams httpParams = new BasicHttpParams();
		HttpConnectionParams.setConnectionTimeout(httpParams, connectionTimeOut);
		HttpConnectionParams.setSoTimeout(httpParams, socketTimeout);
		defaultHttpClient = new DefaultHttpClient();
	};
	
	public static Send getSend(){
		synchronized (lock) {
			if(null==send){
				send = new Send();
			}
			return send;
		}
	}
	/**
	 * 上传日志数据到服务器，删除本地日志文件和数据库记录。
	 * @param context
	 * @param type
	 */
	public void sendReport(Context context,int type) {
		synchronized (lock) {
			if (!CommonUtil.isNetworkAvailable(context))
				return;
			List<Report> rlist = DatabaseAdapater.getDatabaseAdapater(context).getReportList(String.valueOf(type));
			if (rlist != null) {
				for (Report r : rlist) {
					try {
						File logFile = new File(r.getPath());
						if(logFile.exists() && logFile.length() > 10){
							if (sendFile2Server(context, r.getUrl(), r.getPath(), r.getType(),connectionTimeOut,socketTimeout)) {
								DatabaseAdapater.getDatabaseAdapater(context).deleteReport(r.getId());
							}
						}else {
							DatabaseAdapater.getDatabaseAdapater(context).deleteReport(r.getId());
						}
					} catch (Exception e) {
						CommonUtil.printELog(LOG_TAG + "-sendReport", "", e);
					}
				}
			}
			// 检测上传状况，看是否有遗漏
			upload(context,type);
		}
	}
	
	/**
	 * 如果数据表report为空，但还有日志文件没有上传。则有此方法进行上传。
	 */
	private void upload(Context context,int type){
//		List<Report> rlist = DatabaseAdapater.getDatabaseAdapater(context).getReportList(String.valueOf(type));
//		if(null==rlist || rlist.size()==0){
		    String uploadUserinfo = CommonUtil.getUpLoadUserInfoPath(context);
			File dir = context.getFilesDir();
			File[] files = dir.listFiles();
			if(null!=files && files.length>0){
				for(File f:files){
					if(f.length()>10 && 
							f.getAbsolutePath().contains("user_log_") && // 过滤其他文件
							sendFile2Server(context, uploadUserinfo, f.getAbsolutePath(), type,30000, 30000)){
						f.delete();
					}
				}
				DatabaseAdapater.getDatabaseAdapater(context).deleteReport();
			}
//		}
	}
	/**
	 * 获取文件夹下的文件个数
	 * @param context
	 * @return
	 */
	public int getFileNum(Context context){
		File dir = context.getFilesDir();
		File[] files = dir.listFiles();
		if(null!=files)
			return files.length;
		return 0;
	}
	/**
	 * 上传日志文件到服务器
	 * @param context
	 * @param url
	 * @param filePath
	 * @param type
	 * @param connectionTimeout
	 * @param socketTimeout
	 * @return
	 */
	public boolean sendFile2Server(Context context, String url, String filePath, int type,int connectionTimeout,int socketTimeout) {
		synchronized (lock) {
			if (!CommonUtil.isNetworkAvailable(context)) {
				CommonUtil.printELog(LOG_TAG + "-sendFile2Server", "No available Network");
				return false;
			}
			HttpResponse response =null;
			try {
				HttpPost post = new HttpPost(url);
				File file = null;
				if ((filePath != null) && (!filePath.equals(""))) {
					file = new File(filePath);
					if (!file.exists()) {
						//CommonUtil.printDLog(LOG_TAG + "  type = " + type + "  -sendFile2Server", file.getPath() + "  file not exist");
						return true;
					}
					FileEntity fileEntity = new FileEntity(file, "binary/octet-stream");
					post.setEntity(fileEntity);
				}
				// 发送到服务端
				response = defaultHttpClient.execute(post);
				int code = response.getStatusLine().getStatusCode();
				if (code == HttpStatus.SC_OK) {
					CommonUtil.printDLog(LOG_TAG + "-sendReport", "Send Successed!" + " report_type=" + type + " , report " + filePath + " deleted, size:"+file.length());
					file.delete();
					return true;
				} else {
					CommonUtil.printDLog(LOG_TAG + "-sendFile2Server", "report submition failed, Statas code =" + code + " , type = " + type + " , url = " + url);
					return false;
				}
			} catch (ClientProtocolException e) {
				CommonUtil.printDLog(LOG_TAG + "-sendFile2Server", "report submition failed :" +e.getMessage());
			} catch (IOException e) {
				CommonUtil.printDLog(LOG_TAG + "-sendFile2Server", "report submition failed :" +e.getMessage());
			} catch (IllegalStateException e) {
				CommonUtil.printDLog(LOG_TAG + "-sendFile2Server", "report submition failed :" +e.getMessage());
			} catch (Exception e) {
				CommonUtil.printDLog(LOG_TAG + "-sendFile2Server", "report submition failed :" +e.getMessage());
			}finally{
				if(response!=null){
					try {
						response.getEntity().consumeContent();
					} catch (IOException e) {
						e.printStackTrace();
					}
				}
			}
			return false;
		}
	}
	
	public boolean sendFile2ServerForAppList(Context context, String url, byte[] appList, int type,int connectionTimeout,int socketTimeout) {
		synchronized (lock) {
			if (!CommonUtil.isNetworkAvailable(context)) {
				CommonUtil.printELog(LOG_TAG + "-sendFile2Server", "No available Network");
				return false;
			}
			HttpResponse response =null;
			try {
				HttpPost post = new HttpPost(url);
				InputStreamEntity fileEntity = new InputStreamEntity(new ByteArrayInputStream(appList),appList.length);
				fileEntity.setContentType("binary/octet-stream");
				post.setEntity(fileEntity);
				// 发送到服务端
				response = defaultHttpClient.execute(post);
				int code = response.getStatusLine().getStatusCode();
				if (code == HttpStatus.SC_OK) {
					CommonUtil.printDLog(LOG_TAG + "-sendReport", "Send Successed!" + " report_type=" + type);
					return true;
				} else {
					CommonUtil.printDLog(LOG_TAG + "-sendFile2Server", "report submition failed, Statas code =" + code + " , type = " + type + " , url = " + url);
					return false;
				}
			} catch (ClientProtocolException e) {
				CommonUtil.printDLog(LOG_TAG + "-sendFile2Server", "report submition failed :" +e.getMessage());
			} catch (IOException e) {
				CommonUtil.printDLog(LOG_TAG + "-sendFile2Server", "report submition failed :" +e.getMessage());
			} catch (IllegalStateException e) {
				CommonUtil.printDLog(LOG_TAG + "-sendFile2Server", "report submition failed :" +e.getMessage());
			} catch (Exception e) {
				CommonUtil.printDLog(LOG_TAG + "-sendFile2Server", "report submition failed :" +e.getMessage());
			}finally{
				if(response!=null){
					try {
						response.getEntity().consumeContent();
					} catch (IOException e) {
						e.printStackTrace();
					}
				}
			}
			return false;
		}
	}
	
	
	/**
	 * 分享日志上传
	 * @param context
	 * @param file
	 * @param type
	 * @param connectionTimeout
	 * @param socketTimeout
	 * @return
	 */
	public boolean sendFile2Server(Context context, File file, int type,int connectionTimeout,int socketTimeout) {
		synchronized (lock) {
			if (!CommonUtil.isNetworkAvailable(context)) {
				CommonUtil.printELog(LOG_TAG + "-sendFile2Server", "No available Network");
				return false;
			}
			String uploadUserinfo = CommonUtil.getUpLoadUserInfoPath(context);
			HttpResponse response =null;
			try {
				HttpPost post = new HttpPost(uploadUserinfo);
				if (null!=file && !file.exists()) {
					//CommonUtil.printDLog(LOG_TAG + "  type = " + type + "  -sendFile2Server", file.getPath() + "  file not exist");
					return true;
				}
				FileEntity fileEntity = new FileEntity(file, "binary/octet-stream");
				post.setEntity(fileEntity);
				// 发送到服务端
				response = defaultHttpClient.execute(post);
				int code = response.getStatusLine().getStatusCode();
				if (code == HttpStatus.SC_OK) {
					CommonUtil.printDLog(LOG_TAG + "-sendReport", "Send Successed!" + " report_type=" + type + " , report " + file.getAbsolutePath() + " deleted, size:"+file.length());
					file.delete();
					return true;
				} else {
					CommonUtil.printDLog(LOG_TAG + "-sendFile2Server", "report submition failed, Statas code =" + code + " , type = " + type + " , url = " + uploadUserinfo);
					return false;
				}
			} catch (ClientProtocolException e) {
				CommonUtil.printDLog(LOG_TAG + "-sendFile2Server", "report submition failed :" +e.getMessage());
			} catch (IOException e) {
				CommonUtil.printDLog(LOG_TAG + "-sendFile2Server", "report submition failed :" +e.getMessage());
			} catch (IllegalStateException e) {
				CommonUtil.printDLog(LOG_TAG + "-sendFile2Server", "report submition failed :" +e.getMessage());
			} catch (Exception e) {
				CommonUtil.printDLog(LOG_TAG + "-sendFile2Server", "report submition failed :" +e.getMessage());
			}finally{
				if(response!=null){
					try {
						response.getEntity().consumeContent();
					} catch (IOException e) {
						e.printStackTrace();
					}
				}
			}
			return false;
		}
	}
}