package com.easou.users.analysis.mobilephoneInfo;

import java.util.UUID;

import org.json.JSONException;
import org.json.JSONObject;

import android.content.Context;
import android.content.SharedPreferences;
import android.content.SharedPreferences.Editor;
import android.net.wifi.WifiInfo;
import android.net.wifi.WifiManager;
import android.os.Build;
import android.telephony.TelephonyManager;
import android.view.WindowManager;

import com.easou.users.analysis.common.CommonConfig;
import com.easou.users.analysis.common.CommonUtil;
import com.easou.users.analysis.common.OpenUDID;

public class MobilePhoneInfo {
	private final static String	LOG_TAG				= MobilePhoneInfo.class.getName();
	private String				m_IMEI				= null;
	private String				m_IMSI				= null;
	private String				m_APN				= null;
	private String				m_MacAddress		= null;
	private String				m_Firmware_Version	= null;
	private String				m_Buidl_Model		= null;

	//3.0 新增src_code
	private String				m_src_code			= null;
	
	/*
	 * private int m_Screen_Width = -1; private int m_Screen_Height = -1;
	 */
	// 分辨率（height x width）
	private String				m_resolution		= "-1*-1";
	private String				m_Soft_Version		= null;
	private String				m_Soft_Name			= null;

	// 客户端
	private Context				context;
	private static int			isFirstRun;
	private static String		city = "";

	public static String getCity() {
		return city;
	}
	public static void setCity(String city) {
		MobilePhoneInfo.city = city;
	}

	private static final String	IS_FIRST_RUN		= "IS_FIRST_RUN";

	@SuppressWarnings("deprecation")
	public MobilePhoneInfo(Context context) {
		this.context = context;
		@SuppressWarnings("static-access")
		TelephonyManager telephoneManager = (TelephonyManager) context.getSystemService(context.TELEPHONY_SERVICE);

		WindowManager windowManager = (WindowManager) context.getSystemService(Context.WINDOW_SERVICE);
		setSoftVersion(CommonUtil.getVersion(context));
		setSoftName(context.getPackageName());
		setResolution(windowManager.getDefaultDisplay().getHeight() + "*" + windowManager.getDefaultDisplay().getWidth());
		setAPN(telephoneManager.getNetworkOperatorName());
		setIMEI(telephoneManager.getDeviceId());
		setIMSI(getIMSI(context));
		setMacAddress(getMacAddress(context));
		setBuildModel();
		setFirmwareVersion();
		setSrcCode(getSrcCode(context));
	}
	public static String getSrcCode(Context mContext) {
		SharedPreferences mPreferences = mContext.getSharedPreferences(CommonConfig.COMMON_PREFS, Context.MODE_PRIVATE);
		String _keyInPref = mPreferences.getString(CommonConfig.SRC_CODE, "");
		return _keyInPref;
	}
	private void setSrcCode(String srcCode) {
		 m_src_code = srcCode;
	}
	public static String getPhoneNumer(Context context) {
		TelephonyManager phoneMgr=(TelephonyManager)context.getSystemService(Context.TELEPHONY_SERVICE);
		return phoneMgr.getLine1Number();
	}

	private void setIMEI(String imei) {
		m_IMEI = imei;
	}

	private void setIMSI(String imsi) {
		m_IMSI = imsi;
	}

	private void setAPN(String apn) {
		m_APN = apn;
	}

	private void setMacAddress(String macAddress) {
		m_MacAddress = macAddress;
	}
	
	private void setBuildModel() {
		m_Buidl_Model = Build.MODEL;
		if(m_Buidl_Model.equalsIgnoreCase("android")){
			m_Buidl_Model = Build.BRAND;
		}
	}

	private void setFirmwareVersion() {
		m_Firmware_Version = Build.VERSION.RELEASE;
	}

	private void setSoftVersion(String softVersion) {
		m_Soft_Version = softVersion;
	}

	private void setSoftName(String softName) {
		m_Soft_Name = softName;
	}

	public String getIMEI() {
		return m_IMEI;
	}

	public static String getIMEI(Context context) {
		TelephonyManager telephoneManager = (TelephonyManager) context.getSystemService(Context.TELEPHONY_SERVICE);
		return telephoneManager.getDeviceId() == null ? "" : telephoneManager.getDeviceId();
	}

	public String getIMSI() {
		return m_IMSI;
	}

	public String getAPN() {
		return m_APN;
	}

	public String getMacAddress() {
		return m_MacAddress;
	}

	public int isFirstRun() {
		return isFirstRun;
	}

	@SuppressWarnings("static-access")
	public void setFirstRun(int isFirstRun) {
		this.isFirstRun = isFirstRun;
	}

	public static String getCity(Context context) {
//		if (city == null || city.equals("")) {
//			city = CommonUtil.getCity(context);
//		}
		return getCity();
	}

	public static String getMacAddress(Context context) {
		WifiInfo wifiInfo = null;
		SharedPreferences shareData = context.getSharedPreferences(CommonConfig.COMMON_PREFS, Context.MODE_PRIVATE);
		String temp_mac = shareData.getString("mac_address", null);

		if (null == temp_mac) {
			try {
				WifiManager wifiManager = (WifiManager) context.getSystemService(Context.WIFI_SERVICE);
				if (wifiManager.isWifiEnabled()) {
					wifiInfo = (WifiInfo) wifiManager.getConnectionInfo();
				} else {
					wifiManager.setWifiEnabled(true);
					wifiInfo = (WifiInfo) wifiManager.getConnectionInfo();
					wifiManager.setWifiEnabled(false);
				}
				if (wifiInfo != null) {
					temp_mac = wifiInfo.getMacAddress();
				}
			} catch (Exception e) {
				CommonUtil.printELog(LOG_TAG + "-getMacAddress", "", e);
			}

			Editor editor = shareData.edit();
			editor.putString("mac_address", temp_mac);
			editor.commit();
		}
		return temp_mac;
	}

	public static String getIMSI(Context context) {
		TelephonyManager telephoneManager = (TelephonyManager) context.getSystemService(Context.TELEPHONY_SERVICE);
		SharedPreferences shareData = context.getSharedPreferences(CommonConfig.COMMON_PREFS, Context.MODE_PRIVATE);
		String temp_imsi = shareData.getString("imsi", null);
		if (null == temp_imsi) {
			temp_imsi = telephoneManager.getSubscriberId();
			if (temp_imsi != null) {
				Editor editor = shareData.edit();
				editor.putString("imsi", temp_imsi);
				editor.commit();
			}
		}
		return temp_imsi == null ? "" : temp_imsi;
	}

	public String getBuidlModel() {
		return m_Buidl_Model;
	}

	public String getFirmwareVersion() {
		return m_Firmware_Version;
	}

	public String getSoftVersion() {
		return m_Soft_Version;
	}

	public String getSoftName() {
		return m_Soft_Name;
	}

	public String getResolution() {
		return m_resolution;
	}

	public void setResolution(String m_resolution) {
		this.m_resolution = m_resolution;
	}
	
	/**
	 * 根据IMEI生产的id<b>同时传递两个udid，原udid生成方式不变，新加udid2（优先由imsi码生成）</b>
	 * @return
	 */
//	public String genOpenUDID2(String imsi,String imei,String mac){
//		if(null!=imsi && !"".equals(imsi))
//			return OpenUDID.Md5(String.format("%s.%s", "com.easou", imsi));
//		if(null!=imei && !"".equals(imei))
//			return OpenUDID.Md5(String.format("%s.%s", "com.easou", imei));
//		if(null!=mac && !"".equals(mac))
//			return OpenUDID.Md5(String.format("%s.%s", "com.easou", mac));
//		return OpenUDID.Md5(UUID.randomUUID().toString());
//	}
	
	public JSONObject encapsulationInfo() {
		JSONObject map = new JSONObject();
		try {
			// Head
			map.put(CommonConfig.TIME_FLAG, OpenUDID.getTime() == 0 ? System.currentTimeMillis() : OpenUDID.getTime());
			map.put(CommonConfig.TPYE_FLAG, CommonConfig.TYPE_MOBILE_INFO);
			map.put(PhoneParameter.PHONE_NUMBER, MobilePhoneInfo.getPhoneNumer(context));
			map.put(PhoneParameter.PHONE_IMEI, MobilePhoneInfo.getIMEI(context) == null ? "" : MobilePhoneInfo.getIMEI(context));
			map.put(PhoneParameter.PHONE_IMSI, MobilePhoneInfo.getIMSI(context) == null ? "" : MobilePhoneInfo.getIMSI(context));
			map.put(PhoneParameter.PHONE_MAC, MobilePhoneInfo.getMacAddress(context) == null ? "" : MobilePhoneInfo.getMacAddress(context));
			map.put(CommonConfig.APP_KEY, CommonUtil.getAppKey(context));
			map.put(PhoneParameter.PHONE_OPEN_UDID, OpenUDID.getCorpUDID("com.easou") == null ? "" : OpenUDID.getCorpUDID("com.easou"));
			map.put(PhoneParameter.PHONE_OPEN_UDID2,OpenUDID.getCorpUDID2("com.easou") == null ? "" : OpenUDID.getCorpUDID2("com.easou"));
			// Mobile Phone
			map.put(PhoneParameter.PHONE_RESOLUTION, getResolution() == null ? "" : getResolution());
			map.put(PhoneParameter.PHONE_APN, getAPN() == null ? "" : getAPN());
			map.put(PhoneParameter.PHONE_MODEL, getBuidlModel() == null ? "" : getBuidlModel());
			map.put(PhoneParameter.PHONE_FIRMWARE_VERSION, getFirmwareVersion() == null ? "" : getFirmwareVersion());
			map.put(PhoneParameter.PHONE_SOFTVERSION, getSoftVersion() == null ? "" : getSoftVersion());
			map.put(PhoneParameter.PHONE_SOFTNAME, getSoftName() == null ? "" : getSoftName());
			// 客户端
			map.put(CommonConfig.VERSION_CODE_FLAG, CommonConfig.VERSION_CODE == null ? "" : CommonConfig.VERSION_CODE);
			map.put(CommonConfig.CPID_FLAG, CommonUtil.getChannelId(context));
			map.put(CommonConfig.CURRENT_NETWORK_TYPE_FLAG, CommonUtil.getNetworktype(context) == null ? "" : CommonUtil.getNetworktype(context));
			map.put(PhoneParameter.PHONE_CITY, getCity(context) == null ? "" : getCity(context));
			map.put(CommonConfig.OS, CommonConfig.ANDROID_OS);
			map.put(CommonConfig.APP_PATH, CommonUtil.getAppPath(context));
			map.put(CommonConfig.LAST_CPID, CommonUtil.getLastChannelId(context) == null ? "" : CommonUtil.getLastChannelId(context));
			map.put("package_name", getSoftName() == null ? "" : getSoftName());
			map.put("src_code",getSrcCode(context) == null ? "" : getSrcCode(context));
			CommonUtil.printDLog("easou_sdk", "phone_info = " + map.toString());	
		} catch (JSONException e) {
			CommonUtil.printELog(LOG_TAG + "-encapsulationInfo", "", e);
		}
		return map;
	}

	/**
	 * 从配置文件读取是为第一次启动
	 * 
	 * @return
	 */
	public static int isFirstStart(Context context) {
		SharedPreferences ps = context.getSharedPreferences(CommonConfig.COMMON_PREFS, Context.MODE_PRIVATE);
		isFirstRun = ps.getInt(IS_FIRST_RUN, 1);
		if (isFirstRun == 1) {
			changeFirstStart(context);
		}
		return isFirstRun;
	}

	public static int getFirst(Context context){
		SharedPreferences ps = context.getSharedPreferences(CommonConfig.COMMON_PREFS, Context.MODE_PRIVATE);
		isFirstRun = ps.getInt(IS_FIRST_RUN, 1);
		return isFirstRun;
	}
	
	/**
	 * 读完后，改变为不是第一次启动
	 */
	public static void changeFirstStart(Context context) {
		SharedPreferences ps = context.getSharedPreferences(CommonConfig.COMMON_PREFS, Context.MODE_PRIVATE);
		SharedPreferences.Editor editor = ps.edit();
		editor.putInt(IS_FIRST_RUN, 0);
		editor.commit();
	}
}